;include z_romboot
 org 04000h
 db 41h, 42h
 dw rom.init
 ds 12,0
rom.init:
 call cpslt
 jp _main     

;include z_ctrl

rom.ENASLT EQU 0024H
rom.RSLREG EQU 0138H
rom.EXPTBL EQU 0FCC1H


rdslt:
 ex de,hl
 ;DE=PGxx xxxx xxxx xxxx
 rept 5
  srl d;page*2
 endm
 ;DE=xxxx xPGx xxxx xxxx
 CALL rom.RSLREG
 ;A = PRIMARY_SLOT
 ld e,d
 ;A >>= PG*2
rom.rdslt1:
  RRCA
  dec e
  jp nz,rom.rdslt1
 AND    00000011B
 LD c,A;000000Pr
 LD B,0
 LD HL,rom.EXPTBL
 ADD HL,BC
 LD c,A;000000Pr
 LD A,(HL)
 AND 80H;expand flag
 OR c
 bit 7,a
 ret z
 LD c,A;e00000Pr
 rept 4
  INC HL
 endm
 LD A,(HL);exp reg
 ld e,d
rom.rdslt2:
  srl a
  dec e
  jp nz,rom.rdslt2
;000000Ex
 sla a
 sla a
;0000Ex00
 and  00001100b
 OR c;e000ExPr
 ret

cpwork equ 0c000h
cplen equ 01000h
rom equ cpwork+cplen
ram equ rom+1
ramtop equ 8000h

cpslt:
 ld hl,4000h
 call rdslt
 ld (rom),a
 ld hl,cpwork
 call rdslt
 ld (ram),a
 ld hl,rom
 cp (hl)
 ret z
 ld hl,ramtop 
cpslt.loop:
  push hl
  ld a,(rom)
  ld hl,ramtop 
  call rom.ENASLT
  pop hl
  push hl
  ld bc,cplen
  ld de,cpwork
  ldir
  ld a,(ram)
  ld hl,ramtop 
  call rom.ENASLT
  ld hl,cpwork
  pop de
  ld bc,cplen
  ldir
  ex de,hl
  ld a,h
  cp 0c0h
  jr c,cpslt.loop
 ret

;!INCLUDE_ROM
; ds 8000h-$, 0


;end of include z_romboot
;include z_tnu

;include z_spr
;include z_const

;wrt equ 0a2h
dcompr equ 0020H

sp.ini.h  equ 0dch
sp.ini    equ sp.ini.h*256
stksize.h equ 2
stksize   equ stksize.h*256

th.size.h   equ 1
th.size     equ th.size.h*256
th.count equ 32
th.end.h    equ sp.ini.h-stksize.h
th.end      equ th.end.h*256
th.start.h  equ th.end.h-th.count*th.size.h
th.start    equ th.start.h*256

MZO.PSGs    equ th.start
MZO.ch1     equ MZO.PSGs+256
MZO.ch2     equ MZO.ch1+256
MZO.ch3     equ MZO.ch2+256
MZO.chs     equ MZO.ch3+256


th.bottom equ 0

spr.scale equ 1
spr.xmax equ 256<<spr.scale
spr.ymax equ 192<<spr.scale

space.char equ 1

ENASLT EQU 0024H
RSLREG EQU 0138H
EXPTBL EQU 0FCC1H
SETWRT equ 0053H
filvrm equ 0056H
LDIRVM equ 005CH
WRTVDP equ 0047H
RG1SAV equ 0F3E0H
RDVDP  equ 013EH
SNSMAT.a equ 0141h

CHGMOD equ 005FH

IMULT.a equ 3193H;HL ← DE*HL
IDIV.a equ 31E6H;HL ← DE/HL
IMOD.a equ 323AH;HL ← DE mod HL (DE ← DE/HL)

WRTPSG  equ 0093H

CSRY equ 0F3DCH
CSRX equ 0F3DDH

null equ 0

;last.marker: defl $
marker.b macro n
endm

;len.##n: defl $-last.marker
marker.e macro n
endm

;end of include z_const
;include z_th
;include z_ctrl

freeze:
halt
jr freeze

for macro lbend
 ; uses a
 ; c: breaked
#local
 lb:
  rst dcompr; uses a
  jp nc,&lbend
  push HL
  push de
  push bc
  call s
  pop bc
  pop de
  pop HL
  jp c,&lbend
  add HL,bc
  jr lb
 s:
#endlocal
endm

repti macro n,lbend
#local
  push bc
  ld b,&n
 lb:
  push bc
  call s
  pop bc
  jr c,lbend2
  djnz lb
  lbend2:
  pop bc
  jp &lbend
 s:
#endlocal
endm


reptb macro lbend
#local
 inc b
 djnz lb
 jp &lbend
 lb:
  push bc
  call s
  pop bc
  jp c,&lbend
 djnz lb
 jp &lbend
 s:
#endlocal
endm



iff.nz equ 0
iff.z  equ 1
iff.nc equ 2
iff.c  equ 3

iff macro cnd,to
 if iff.&cnd eq iff.nz
  jr z,&to
 endif
 if iff.&cnd eq iff.z
  jr nz,&to
 endif
 if iff.&cnd eq iff.nc
  jr c,&to
 endif
 if iff.&cnd eq iff.c
  jr nc,&to
 endif
 ;jr cnd, skip
 ;jr to
 ;skip:
endm

break macro
 scf
 ret
endm
break.if macro cnd
#local
 iff &cnd ,jj
  break
 jj:
#endlocal
endm

continue macro
 or a
 ret
endm
continue.if macro cnd
#local
 iff &cnd,jj
  continue
 jj:
#endlocal
endm


djnzr macro reg,j
 dec &reg
 jr nz,&j
endm

callhl macro
 push hl
 ld hl,$+5
 ex (sp),hl
 jp (hl); call hl
endm

stride macro lim,to
 if lo($)<&lim
  exitm
 endif
 ds 256+&to-lo($),0cdh
endm


unreach macro mesg
 ;trace mesg
 call h.unreach
endm


; unreach &lb
head macro lb
 marker.b &lb
 &lb:
endm

defsub macro n
 head &n
endm
endsub macro n
 ret
 marker.e &n
endm
deftbl macro n
 head &n
endm
endtbl macro n
 marker.e &n
endm


defsub h.unreach.default
 ld a,'u'
 ld hl,1800h
 call 4dh
 dw 0x18,0xfe
endsub h.unreach.default

h.nop:
 ret
 
sethook macro h.name, adr
 ld (&h.name+1),&adr
endm

;end of include z_ctrl
;include z_sp
;include z_mem



movw macro dst,src
 push hl
 ld hl,&src
 ld &dst,hl
 pop hl
endm

popa macro xx
  ex (sp),hl
  ld a,h
  pop HL
endm

pushall macro
 push af
 push bc
 push de
 push hl
endm
popall macro
 pop hl
 pop de
 pop bc
 pop af
endm


pushi macro n,rp
 ld &rp,&n
 push &rp
endm
const macro n,reg
 ld (&n-2),&reg
endm
ldconst macro reg,n
 ld &reg,0
 &n:
endm
peekconst macro reg,n
 ld &reg,(0)
 &n:
endm

;end of include z_mem
;include z_debug
;include z_math


;16bit shifts
slhl macro xx
 sla l
 rl h
endm
srahl macro xx
 sra h
 rr l
endm
srlhl macro xx
 srl h
 rr l
endm
slde macro xx
 sla e
 rl d
endm
srade macro xx
 sra d
 rr e
endm
srlde macro xx
 srl d
 rr e
endm
slbc macro xx
 sla c
 rl b
endm
srabc macro xx
 sra b
 rr c
endm
srlbc macro xx
 srl b
 rr c
endm



subhl macro rp
 and a
 sbc hl,&rp
endm

cpde.a macro n
 rst dcompr
endm


defsub xrnd.a
; h l d e
 ld hl,(xrnd.seed+2)
 ld de,(xrnd.seed)
 ; s[0] ^= s[0] << 13
 push de
 pop bc
; h l d e
; b c
 ld a,l
 rept 3
  srl a
  rr b
  rr c
 endm
 ld a,h
 xor b
 ld h,a
 ld a,l
 xor c
 ld l,a
 ; s[0] ^= s[0] >> 17
 
 push hl
 pop bc
; h l d e
;     b c
 srl b
 rr c
 
 ld a,d
 xor b
 ld d,a
 ld a,e
 xor c
 ld e,a

 ; s[0] ^= s[0] << 5;
 ; h l d e
 ;   b c
 ld b,l
 ld c,d
 ld a,e
 rept 3
  srl a
  rr b
  rr c
 endm

 ld a,l
 xor b
 ld l,a
 ld a,d
 xor c
 ld d,a
 
 ld (xrnd.seed+2),hl
 ld (xrnd.seed),de

 ret
endsub xrnd.a


defsub rnd
 push af
 call rnd.a
 pop af
endsub rnd

defsub rnd.a
 ex de,hl
 ld hl,07fffh
 call udiv.a
 inc bc
 push bc
 call xrnd.a
 res 7,h
 pop de
 call udiv.a
 ld h,b
 ld l,c
endsub rnd.a

defsub abs
 bit 7,h
 ret z
neghl:
 ld de,0
 ex de,hl
 subhl de
 ret
endsub abs

defsub clamp
#local
 ; hl value
 ; bc min
 ; de max
 push hl
 subhl bc
 bit 7,h
 pop hl
 jr z, notmin
  ld h,b
  ld l,c
  ret
 notmin:
 push hl
 subhl de
 bit 7,h
 pop hl
 ret nz
 ex de,hl
 ret
#endlocal
endsub clamp

defsub mul.a
 ; hl*=de
#local
 ld b,h
 ld c,l
 
 ld hl,0
loop:
 srlbc 0
 jr nc,notad
 add hl,de
notad:
 slde 0
 ld a,b
 or c
 jr nz,loop
 
#endlocal
endsub mul.a

defsub log.a
#local
 push hl
 ld a,16
loop:
 dec A
 slhl 0
 jr nc,loop
 pop hl
#endlocal
endsub log.a



neghl.sv macro xx
 push de
 call neghl
 pop de
endm
negde.sv macro xx
 ex de,hl
 neghl.sv 0
 ex de,hl
endm


defsub div.a
; hl/=de with signed
; de=remainder
#local
 ld a,0
 ; hl=abs(hl), 
 ; a++ if hl<0
 bit 7,h
 jr z, hposi
 xor 3
 neghl.sv 0
hposi:
 ; de=abs(de), 
 ; a++ if de<0
 bit 7,d
 jr z, dposi
 xor 1
 negde.sv 0
dposi:
 push af
 call udiv.a
 ex de,hl; remainder
 ld h,b
 ld l,c
 pop af
 bit 0,a
 jp z, nnq
; negate quotient
 neghl.sv 0
nnq:
 bit 1,a
 jp z, nnr
; negate remainder
 negde.sv 0
nnr:
 
#endlocal
endsub div.a

defsub udiv.a
 ; bc=hl/de
 ; hl=hl%de
#local
 ld a,d
 or e
 ret z; /0 error
 
 ld b,0
bitc.lp:
 slde 0
 inc b
 rst dcompr
 jr nc, bitc.lp
bitc.br:
 srlde 0
 ld a,b
 ld bc,0
 ;ld (hexval),de
 ;call showhex
put.lp:
 subhl de
 bit 7,h
 jr z, put1
put0:
 add hl,de
 and a
 jr puten
put1:
 and a
 scf  
puten:
 rl c
 rl b
 srlde 0
 dec a
 jr nz, put.lp 
put.br:

#endlocal
endsub udiv.a


;end of include z_math


;debug
show macro reg
 ld (hexval),&reg
 call showhex
endm
showm macro ad
 push hl
 ld HL,(&ad)
 show HL
 pop HL
endm
showlb macro lb
 push hl
 ld hl,&lb
 ld (hexval),hl
 call showhex
 pop hl
endm

trace macro v
 push af
 ld a,&v
 ld (trad),a
 pop af
 call trace.s
endm


defsub showhex
#local
 push af
 push bc
 push HL
 ld hl,(hexval)
 ld b,4
 loop:
  xor a
  rept 4
   slhl 0
   rla
endm
  call showhex1
 djnz loop
 ld a,32
 call wrt
 pop HL
 pop bc
 pop af
 ret
#endlocal
endsub showhex

defsub showhex1
#local
 cp 10
 jp nc, els
 add a,48
 jp wrt
 els:
 add a,65-10
 jp wrt
#endlocal
endsub showhex1


defsub abort
 call wrt
 db 018h,0feh
endsub abort

defsub trace.s
 push af
 push hl
 ld a,(trad)
 ld hl,1ae0h
 call wrt
 call 4dh
 inc a
 ld (trad),a
 ld a,32
 call wrt
 pop hl
 pop af
endsub trace.s

defsub showz
 push af
 jr z,showz.s
 ld a,'N'
 call wrt
 showz.s:
 ld a,'z'
 call wrt
 ld a,32
 call wrt
 pop af
endsub showz


defsub showc
 push af
 jr c,showc.s
 ld a,'N'
 call wrt
 showc.s:
 ld a,'c'
 call wrt
 ld a,32
 call wrt
 pop af
endsub showc


defsub wrt
#local
 push hl
 push af
 ld hl,(cursor)
 call 4dh
 inc hl
 ld a,h
 cp 1bh
 jr c,sk
  ld h,18h
 sk:
 ld (cursor),hl
 pop af
 pop hl
 ret
#endlocal
endsub wrt

defsub print.a
 ld a,(hl)
 cp 0
 ret z
 call wrt
 inc hl
 jr print.a
endsub print.a

defsub unreach.new
 ld hl,unreach.m
 call print.a
 pop hl
 show hl
 db 18h,0feh
 unreach.m:
 db "unreach"
 db 0
endsub unreach.new
unreach.new.init macro xx
 ld hl, unreach.new
 sethook h.unreach, hl
endm
;end of include z_debug
sp.get macro
 ld HL,0
 ADD hl, sp
endm
sp.set macro
 ld sp,hl
endm


showsp macro
 ld (sptmp),sp
 showm sptmp
endm


showstk macro
 showsp
 ld (sva),a
 ld a,":"
 call wrt
 ld a,(sva)
 ex (sp),hl
 show hl
 ex (sp),hl
endm



;end of include z_sp
;include z_vdp
;https://www.msx.org/wiki/VDP_Status_Registers
;st 0 bit 7
;read 1

;https://www.msx.org/wiki/VDP_Mode_Registers
;ctrl 1 bit 5 set 0


;include th

defsub susint
 ld a,(RG1SAV)
 res 5,a
 ld b,A
 ld c,1
 jp WRTVDP
endsub susint
defsub rstint
 ld a,(RG1SAV)
 set 5,a
 ld b,A
 ld c,1
 jp WRTVDP
endsub rstint
defsub inted
 call RDVDP
 bit 7,a
endsub inted

doint:
 call inted
 jr z, norst
 ld hl,timecnt
 inc (hl)
 call h.tntimi
 norst:
 ;call rstint
 ret


defsub setInterval
 sethook h.tntimi, hl
endsub setInterval
defsub clearInterval
 ld hl, int.nop
 jp setInterval
endsub clearInterval

defsub int.nop
ret
endsub int.nop


vdptest macro
#local
stk1:
 ds 256,35
stk2:
 ds 256,42
stk3:

vl:
 call susint
 ld sp,stk2
 ld hl,stk1
 ld de,1800h
 ld bc,256
 call LDIRVM


 ld sp,stk3
 call doint
 ld hl,stk2
 ld de,1900h
 ld bc,256
 call LDIRVM
 jp vl
#endlocal
endm

screen1:
 ld a,1
 call CHGMOD
 ld a,(RG1SAV)
 set 1,a
 ld b,A
 ld c,1
 call WRTVDP
 ret

defsub screen2
 ;ld a,2
 ;call CHGMOD
 call 72h
 ld a,(RG1SAV)
 set 1,a
 ld b,A
 ld c,1
 call WRTVDP
endsub screen2

;end of include z_vdp




th.ofs.stp equ 256-4
th.ofs.sp equ th.ofs.stp+1
th.ofs.spini equ th.ofs.stp
fld.top equ th.ofs.spini-2
th.st.blank equ 0c9h
th.st.active equ 31h

th.for.a macro nx,st,en
#local
 ld a,&st+(th.start.h)
 loop:
  cp &en+(th.start.h)
  jp nc, &nx
  push af
  ld h,a
  ld l,0
  call do
  popa 0
  ld h,a
  ld l,0
  jp c, &nx
  inc a
 jr loop
 do:
#endlocal
endm

th.new.range macro st,en
 ld bc, 0 + ((&en+th.start.h)<<8) + (&st+th.start.h)
 ld(th.new.start),bc
endm

defsub th.isblank.a
 ; h= thread
 ; z if true
 ld l, th.ofs.stp
 ld a,(hl)
 cp th.st.blank
endsub th.isblank.a


defsub th.find.blank
; h: result  nz: not found
#local
 ld a,(th.new.end)
 ld b,a
 ld a,(th.new.start)
 ld h,a
 ld l,th.ofs.stp
loop:
 ld a,(hl)
 cp th.st.blank
 ret z
 ld a,b
 inc h
 cp h
 jr nz,loop
 inc a; reset z
#endlocal
endsub 0

defsub th.new
; nz for alloc fail
#local
 call th.find.blank
 ret nz
 ; TODO th.ofs.stp
 ld L,th.ofs.stp
 ld (HL),31h
 inc HL
 ld (HL),th.ofs.spini
 ld a,h
 inc HL
 ld (hl),a
 inc HL
 ld (HL),0c9h
 ld l,th.bottom
 ld a,l
 cp l; set z
 ret
#endlocal
endsub th.new

defsub th.init
#local
 th.for.a lbend, 0, th.count
  ; TODO th.ofs.stp
  ld L, th.ofs.stp
  ld (HL),th.st.blank
  continue
 lbend:
 ; disable timer
 ld HL,0fd9fh
 ld (hl),0c9h
 call susint
 ret
#endlocal
endsub th.init

defsub th.stepall
 th.for.a thnx, 0, th.count
  ;todo th.ofs.stp
  ld (th.cur),hl
  call th.isblank.a
  continue.if z
  call th.step

  ; chk reserved kill
  ld hl,0
  ld (th.cur),hl
  ld a,(th.kill.reserve)
  cp 0
  continue.if z
  ld h,a
  call th.kill.a
  xor a
  ld (th.kill.reserve),a
  continue
 thnx:
endsub th.stepall

defsub th.kill.a
; th.term = self kill
; th.kill = kill from either external or internal
#local
 ; h: th
 ld a,(th.cur+1)
 cp h
 jr z, reserve
  ld l, th.ofs.stp
  ld (hl), th.st.blank
  ret
 reserve:
  ld (th.kill.reserve),a
#endlocal
endsub th.kill.a


defsub th.step
 ld (adrssp+1),sp
 ld HL,(th.cur)
 ld l,th.ofs.stp
 ;call susint
 jp (hl)
endsub th.step


defsub th.term
 ld hl,(th.cur)
 ; TODO th.ofs.stp
 ld L,th.ofs.stp
 ld (hl),th.st.blank
 jp adrssp
endsub th.term

th.with.do macro to
#local
 th.with pr
 jr &to
 pr:
#endlocal
endm

th.with.setdst macro reg
 ld (th.jpdest-2),&reg
endm
th.with macro pr
 movw (th.jpdest-2), &pr
 call th.with.s
endm
th.with.ret macro n
 jp th.ewith
endm



th.pushi macro val
 ld bc,&val
 call th.push
endm


;end of include z_th

;include z_oop


;include z_assert




a.reg.trc:
dw 0
a.reg.adr:
dw 0
a.reg.min:
dw 0
a.reg.val:
dw 0
a.reg.max:
dw 0
a.regi macro n,v
 push hl
 ld hl,&v
 ld (a.reg.&n),hl
 pop hl
endm
a.regr macro n,v
 ld (a.reg.&n),&v
endm

a.dummy macro
#local
#endlocal
endm



getpc:
 pop hl
 push hl
 ret



;end of include z_assert

;a2 a1  oldpc oldix lcl1 lcl2
argidx equ 2
getarg.hl macro n
 getarg &n
endm
getarg macro n
 ld l,(ix+argidx+&n*2)
 ld h,(ix+argidx+&n*2+1)
endm
getarg.de macro n
 ld e,(ix+argidx+&n*2)
 ld d,(ix+argidx+&n*2+1)
endm
setarg macro n
 ld (ix+argidx+&n*2),l
 ld (ix+argidx+&n*2+1),h
endm

setlcl macro n
 ld (ix-(&n*2-1)),h
 ld (ix-&n*2),l
endm

getlcl.hl macro n
 getlcl &n
endm
getlcl macro n
 ld h,(ix-(&n*2-1))
 ld l,(ix-&n*2)
endm
getlcl.de macro n
 ld d,(ix-(&n*2-1))
 ld e,(ix-&n*2)
endm

addarg macro
 push hl
; hl=arg  stktp=af
;ex (sp),hl
;ld a,h
;push af
endm



pusharg macro n
 getarg &n
 push HL
endm

pushlcl macro n
 getlcl &n
 push HL
endm


pops macro n
 rept &n*2
  inc sp
endm
endm



pushthis macro xx
 getthis &xx
 push af
endm
popthis macro xx
 popa &xx
 ld (this),a
endm


invoketg.a.rom macro fld
 ld a,h
 ld (this),a
 getfld.hl.rom &fld
 callhl
endm
invoke.rom macro fld
 getfld.hl.rom &fld
 callhl 
 getthis 0
endm


getfld.hl.rom macro fld
 ld h,a
 ld l,&fld
 ld(getfld.hl.rom.fld),hl
 call h.getfld.hl.rom
endm
getfld.de.rom macro fld
 ld d,a
 ld e,&fld
 ld(getfld.de.rom.fld),de
 call h.getfld.de.rom
endm
setfld.rom macro fld
 ld d,a
 ld e,&fld
 ld(setfld.rom.fld),de
 call h.setfld.rom
endm
getfldtg.rom macro n
;hl=tg
 ld l,&n
 ld(getfld.hl.rom.fld),hl
 call h.getfld.hl.rom
endm

setfldtg.rom macro n
; stk=val hl=tg
 ld l,&n
 ld (setfld.rom.fld),hl
 pop hl
 call h.setfld.rom
endm




curth2this macro
 ld a,(th.cur+1)
 ld (this),a
endm
getthis macro xx
 ld a,(this)
endm

new macro Class,flds,st,en
 th.new.range &st, &en
 pushi &flds, bc
 pushi &Class, bc
 call o.new
endm


new.arg.i macro n,v
 ld hl,&v
 new.arg &n
endm
new.arg macro n
 push hl
 pushi &n,bc
endm


fld.def macro n
 &n equ fldidx
 fldidx:defl fldidx-2
endm
class macro Class,super
 unreach "c"
 marker.b 0
 dw &super
 fldidx:defl fld.top
 &Class:
  fld .class,&Class
  fld .statusReg,0
endm
fld.bottom macro Class
 if 0
  if bottomOf&Class ne fldidx
   .error bottom ne fldidx
  endif
 else
 bottomOf&Class:defl fldidx
 endif
endm
fld macro n,v
 if 0
  if &n ne fldidx
   .error &n ne fldidx
  else
   fldidx:defl fldidx-2
  endif
 else
  fld.def &n
 endif
 pushi &v,bc
endm
unuse macro xx
 fldidx:defl fldidx-2
 pushi 0,bc
endm
meth macro Class,n
 fld .&n, &Class.&n
endm
met2 macro Class,n
 fld &n, &Class&n
endm

class Object,null
 fld .main,null
 fld.bottom Object
 marker.e Object



yield macro
 pushthis 0
 push ix
 call th.yield
 pop ix
 popthis 0
endm

def.noent: defl 0
def macro n,args,lcls
head &n
 def.args:defl &args
 def.locals:defl &lcls
 if def.noent eq 0
  enter &lcls
 endif
endm
enter macro locals
 ;  arg2 arg1 retad oldix lcl1 lcl2
 push ix
 ld ix,0
 add ix,sp
 rept &locals
  push HL
endm
endm
enddef macro n
 if def.noent eq 0
  exit def.args
 else
  ret
 endif
 marker.e &n
endm
exit macro n
 ld sp,ix
 pop ix
 if &n!=0
  exx
  pop bc
  pops &n
  push bc
  exx
 endif
 ret
endm


defsub isobj.a
 ;hl=obj?
 ;cy=true
 ld a,h
 cp th.start.h
 jr c,notobj
 cp th.end.h
 jr nc,notobj
 scf
 ret
 notobj:
 and a
endsub isobj.a


defsub is.subclass.a
#local
 ; hl=Subclass
 ; de=Superclass
 ; z:true
 top:
 cpde.a 0
 ret z
 call get.superclass
 push de
 ld de,null
 cpde.a 0
 pop de
 jr nz,top
 cpde.a 0
#endlocal
endsub is.subclass.a

;end of include z_oop
;include z_sub

;end of include z_sub

class Sprite,Object
 fld .main, 0
 fld.bottom Object
 fld .x, 100
 fld .y, 100
 fld .p, 0
 fld .c, 2
 fld.bottom Sprite
 marker.e Sprite

outwrt macro n
  out (98h),a
endm


spr.unscale macro n
 ; HL -> A
 rept spr.scale
  srlhl 0
endm
 LD A,L
endm



defsub spr.putslp
#local
 ld hl, 1b00h
 call SETWRT
 ld a, (spr.puts.b)
 ld b, th.count
 sprl:
  ld (this),a
  call spr.put1
  getthis 0
  ld c,a
  ld a,(spr.puts.s)
  add a,c
  cp th.end.h
  jr c,nrst
   sub th.count
  nrst:
 djnz sprl
 ld a,208
 outwrt 0
#endlocal
endsub spr.putslp




;end of include z_spr
;include z_bool


true equ -1
false equ 0

ret.true.if macro flg
#local
 iff &flg ,tru
  ld hl,true
  ret
 tru:
#endlocal
endm

ret.false.if macro flg
#local
 iff &flg ,tru
  ret.false
 tru:
#endlocal
endm

ret.false macro
 ld hl,false
 ret
endm

hleqde:
 subhl de
 ret.true.if z
 ret.false

hlnede:
 subhl de
 ret.true.if nz
 ret.false

hlgtde:
 subhl de
 ret.false.if z
 bit 7,h
 ret.true.if z
 ret.false

hlltde:
 subhl de
 bit 7,h
 ret.true.if nz
 ret.false

hlgede:
 subhl de
 ret.true.if z
 bit 7,h
 ret.true.if z
 ret.false

hllede:
 subhl de
 ret.true.if z
 bit 7,h
 ret.true.if nz
 ret.false

ziffalse:
#local
 push af
 call ziffalse.a
 popa 0
 ret
ziffalse.a:
 ld a,0
 cp h
 ret nz
 cp l
 ret
#endlocal

jpf macro to
 call ziffalse
 jp z,&to
endm

andand macro fls
 jpf &fls
endm
oror macro tru
 call ziffalse
 jp nz,&tru
endm

bitand.a macro xx
 ; hl&=de
 ld a,h
 and d
 ld h,a
 ld a,l
 and e
 ld l,a
endm

bitor.a macro xx
 ; hl|=de
 ld a,h
 or d
 ld h,a
 ld a,l
 or e
 ld l,a
endm


flagtobool macro fl
#local
 jr &fl, yes
  ld hl,false
  jr skp
 yes:
  ld hl,true
 skp:
#endlocal
endm

defsub nothl
#local
 jpf fls
  ld hl, false
  ret
 fls:
  ld hl, true
#endlocal
endsub nothl

;end of include z_bool
;include z_key


defsub keyall
;show hl
#local
 ld hl,keymat1
 ld de,keymat2
 ld bc,13
 ldir
 ld a,(keyuse)
 and a
 ret z
 ld a,0
 ld hl,keymat1
 lp:
 push af
 call SNSMAT.a
 xor 255
 ld (hl),a
 pop af
 inc hl
 inc a
 cp 11
 jr c,lp
 ld a,1
 call snstk.a
 ld (hl),d
 inc hl
 ld a,2
 call snstk.a
 ld (hl),d
#endlocal
endsub keyall
defsub snstk.a
#local
 push hl
  ld e,a
  push de
   call 0d5h
  pop de
  ld c,a
  ld b,0
  ld hl,trig2mat
  add hl,bc
  ld d,(hl)
  ld a,e
  push de
   call 0d8h
  pop de
  cp 0
  jr z,skp1
  set 0,d
  skp1:
  ld a,e
  inc a
  inc a
  push de
   call 0d8h
  pop de
  cp 0
  jr z,skp2
  set 1,d
  skp2:
 pop hl
#endlocal
endsub snstk.a



defsub getkey.a
ex de,hl
ld hl,keymat1
call chkmat
ld hl,0
ret z
ld hl,keymat2
call chkmat
ld hl,1
ret z
inc hl
endsub getkey.a

defsub chkmat
push de
ld a,d
ld d,0
add hl,de
and (hl)
pop de
endsub chkmat

defsub getkey
push af
call getkey.a
pop af
endsub getkey

defsub stickX.a
#local
; a :stickid
; hl:  vec norm
 cp 0
 jr nz,nx1
  ld a,(keymat1+8)
  bit 7,a
  ret nz
  bit 4,a
  jp nz,neghl
  ld hl,0
  ret
 nx1:
 cp 1
 jr nz,nx2
  ld a,(keymat1+11)
  bit 7,a
  ret nz
  bit 4,a
  jp nz,neghl
  ld hl,0
  ret
 nx2:
 cp 2
 jr nz,nx3
  ld a,(keymat1+12)
  bit 7,a
  ret nz
  bit 4,a
  jp nz,neghl
  ld hl,0
  ret
 nx3:
 ; w 5-b4  a 2-b6  s 5-b0  d 3-b1
  ld a,(keymat1+3);d
  bit 1,a
  ret nz
  ld a,(keymat1+2);a
  bit 6,a
  jp nz,neghl
  ld hl,0
#endlocal
endsub stickX.a


defsub stickY.a
#local
; a :stickid
; hl:  vec norm
 cp 0
 jr nz,nx1
  ld a,(keymat1+8)
  bit 6,a
  ret nz
  bit 5,a
  jp nz,neghl
  ld hl,0
  ret
 nx1:
 cp 1
 jr nz,nx2
  ld a,(keymat1+11)
  bit 6,a
  ret nz
  bit 5,a
  jp nz,neghl
  ld hl,0
  ret
 nx2:
 cp 2
 jr nz,nx3
  ld a,(keymat1+12)
  bit 6,a
  ret nz
  bit 5,a
  jp nz,neghl
  ld hl,0
  ret
 nx3:
 ; w 5-b4  a 2-b6  s 5-b0  d 3-b1
  ld a,(keymat1+5);s
  bit 0,a
  ret nz
  bit 4,a ; w
  jp nz,neghl
  ld hl,0
#endlocal
endsub stickY.a



;end of include z_key
;include z_mzo
;range 0-20
def MZO.main,0,0
 #local
 lbexit:
 #endlocal
 enddef MZO.main
MZO.init:
pop bc
pop hl
ld (MZO.init.ch),hl
pop hl
ld (MZO.init.mp),hl
push bc
#local
;"static all";
;$len[ch]=20160/2;
ld hl,10080; const val
push hl
ld hl,(MZO.init.ch)
add hl,hl
ld de,MZOgbl_len
add hl,de
pop de
ld (hl),e
inc hl
ld (hl),d
;$mcount[ch]=0;
ld hl,0; const val
push hl
ld hl,(MZO.init.ch)
add hl,hl
ld de,MZOgbl_mcount
add hl,de
pop de
ld (hl),e
inc hl
ld (hl),d
;$tempoc[ch]=336;
ld hl,336; const val
push hl
ld hl,(MZO.init.ch)
add hl,hl
ld de,MZOgbl_tempoc
add hl,de
pop de
ld (hl),e
inc hl
ld (hl),d
;$mute[ch]=0;
ld hl,0; const val
push hl
ld hl,(MZO.init.ch)
add hl,hl
ld de,MZOgbl_mute
add hl,de
pop de
ld (hl),e
inc hl
ld (hl),d
;$mpoint[ch]=mp;
ld hl,(MZO.init.mp)
push hl
ld hl,(MZO.init.ch)
add hl,hl
ld de,MZOgbl_mpoint
add hl,de
pop de
ld (hl),e
inc hl
ld (hl),d
lbexit:
#endlocal
ret
MZO.stop:
pop bc
pop hl
ld (MZO.stop.ch),hl
push bc
#local
;"static all";
;$mpoint[ch]=0;
ld hl,0; const val
push hl
ld hl,(MZO.stop.ch)
add hl,hl
ld de,MZOgbl_mpoint
add hl,de
pop de
ld (hl),e
inc hl
ld (hl),d
;var chn=clamp(ch,0,2)
ld hl,2; const val
push hl
ld hl,0; const val
push hl
ld hl,(MZO.stop.ch)
pop bc
pop de
call clamp
ld (MZO.stop.chn),hl
;if !$mute[ch]
ld hl,(MZO.stop.ch)
add hl,hl
ld de,MZOgbl_mute; const val
add hl,de
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
call nothl
jpf lb4;then
;sound_vol(chn,0);
ld hl,0; const val
ld e,l
push de
ld hl,(MZO.stop.chn)
ld a,l
pop de
add a,8
call WRTPSG
getthis 0
lb4:
lbexit:
#endlocal
ret
MZO.step:
pop bc
pop hl
ld (MZO.step.ch),hl
push bc
#local
;"static all";
;var chn=clamp(ch,0,2)
ld hl,2; const val
push hl
ld hl,0; const val
push hl
ld hl,(MZO.step.ch)
pop bc
pop de
call clamp
ld (MZO.step.chn),hl
;var mpoint=$mpoint[ch]
ld hl,(MZO.step.ch)
add hl,hl
ld de,MZOgbl_mpoint; const val
add hl,de
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
ld (MZO.step.mpoint),hl
;if !mpoint
ld hl,(MZO.step.mpoint)
call nothl
jpf lb5;then
;return return;
jp lbexit
lb5:
;var mcount=$mcount[ch]
ld hl,(MZO.step.ch)
add hl,hl
ld de,MZOgbl_mcount; const val
add hl,de
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
ld (MZO.step.mcount),hl
;var tempoc=$tempoc[ch]
ld hl,(MZO.step.ch)
add hl,hl
ld de,MZOgbl_tempoc; const val
add hl,de
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
ld (MZO.step.tempoc),hl
;var len=$len[ch]
ld hl,(MZO.step.ch)
add hl,hl
ld de,MZOgbl_len; const val
add hl,de
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
ld (MZO.step.len),hl
;var v=$v [ch]
ld hl,(MZO.step.ch)
add hl,hl
ld de,MZOgbl_v; const val
add hl,de
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
ld (MZO.step.v),hl
;var cmd
;if !$mute[ch]
ld hl,(MZO.step.ch)
add hl,hl
ld de,MZOgbl_mute; const val
add hl,de
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
call nothl
jpf lb6;then
;sound_vol(chn,v);
ld hl,(MZO.step.v)
ld e,l
push de
ld hl,(MZO.step.chn)
ld a,l
pop de
add a,8
call WRTPSG
getthis 0
lb6:
;mcount-=tempoc;
ld hl,(MZO.step.mcount)
ld de,(MZO.step.tempoc)
subhl de
ld (MZO.step.mcount),hl
;if v>0
ld hl,0; const val
push hl
ld hl,(MZO.step.v)
pop de
call hlgtde
jpf lb7;then
;v--;
ld hl,(MZO.step.v)
dec hl
ld (MZO.step.v),hl
lb7:
lb8:
;while mcount>=-tempoc && mcount<=0
ld hl,(MZO.step.tempoc)
call neghl
push hl
ld hl,(MZO.step.mcount)
pop de
call hlgede
andand lb10
ld hl,0; const val
push hl
ld hl,(MZO.step.mcount)
pop de
call hllede
lb10:
jpf lb9
;cmd=peek(mpoint);
ld hl,(MZO.step.mpoint)
ld l,(hl)
ld h,0
ld (MZO.step.cmd),hl
;if cmd>=0 && cmd<96
ld hl,0; const val
push hl
ld hl,(MZO.step.cmd)
pop de
call hlgede
andand lb12
ld hl,96; const val
push hl
ld hl,(MZO.step.cmd)
pop de
call hlltde
lb12:
jpf lb11;then
;if !$mute[ch]
ld hl,(MZO.step.ch)
add hl,hl
ld de,MZOgbl_mute; const val
add hl,de
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
call nothl
jpf lb13;then
;sound_scale(chn,cmd);
ld hl,(MZO.step.cmd)
push hl
ld hl,(MZO.step.chn)
ld a,l
pop hl
call mus.scale.a
getthis 0
lb13:
;v=12;
ld hl,12; const val
ld (MZO.step.v),hl
;mpoint++;
ld hl,(MZO.step.mpoint)
inc hl
ld (MZO.step.mpoint),hl
;if peek(mpoint)==123
ld hl,123; const val
push hl
ld hl,(MZO.step.mpoint)
ld l,(hl)
ld h,0
pop de
call hleqde
jpf lb15;then
;mpoint++;
ld hl,(MZO.step.mpoint)
inc hl
ld (MZO.step.mpoint),hl
;mcount+=peekw(mpoint);
ld hl,(MZO.step.mpoint)
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
push hl
ld hl,(MZO.step.mcount)
pop de
add hl, de
ld (MZO.step.mcount),hl
;mpoint+=2;
ld hl,(MZO.step.mpoint)
ld de,2; const val
add hl, de
ld (MZO.step.mpoint),hl
jp lb14
lb15:;else
;mcount+=len;
ld hl,(MZO.step.mcount)
ld de,(MZO.step.len)
add hl, de
ld (MZO.step.mcount),hl
lb14:
jp lb9
lb11:
;switch cmd
ld hl,(MZO.step.cmd)
ld de,99; const val
rst dcompr
jp z,lb160
ld de,124; const val
rst dcompr
jp z,lb161
ld de,104; const val
rst dcompr
jp z,lb162
ld de,255; const val
rst dcompr
jp z,lb163
ld de,100; const val
rst dcompr
jp z,lb164
ld de,101; const val
rst dcompr
jp z,lb165
ld de,102; const val
rst dcompr
jp z,lb166
ld de,103; const val
rst dcompr
jp z,lb167
ld de,107; const val
rst dcompr
jp z,lb168
ld de,110; const val
rst dcompr
jp z,lb169
ld de,115; const val
rst dcompr
jp z,lb1610
ld de,116; const val
rst dcompr
jp z,lb1611
ld de,118; const val
rst dcompr
jp z,lb1612
ld de,120; const val
rst dcompr
jp z,lb1613
ld de,122; const val
rst dcompr
jp z,lb1614
getthis 0
jp lb16def
lb160:
getthis 0
;case 99
;v=0;
ld hl,0; const val
ld (MZO.step.v),hl
;mpoint++;
ld hl,(MZO.step.mpoint)
inc hl
ld (MZO.step.mpoint),hl
;if peek(mpoint)==123
ld hl,123; const val
push hl
ld hl,(MZO.step.mpoint)
ld l,(hl)
ld h,0
pop de
call hleqde
jpf lb18;then
;mpoint++;
ld hl,(MZO.step.mpoint)
inc hl
ld (MZO.step.mpoint),hl
;mcount+=peekw(mpoint);
ld hl,(MZO.step.mpoint)
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
push hl
ld hl,(MZO.step.mcount)
pop de
add hl, de
ld (MZO.step.mcount),hl
;mpoint+=2;
ld hl,(MZO.step.mpoint)
ld de,2; const val
add hl, de
ld (MZO.step.mpoint),hl
jp lb17
lb18:;else
;mcount+=len;
ld hl,(MZO.step.mcount)
ld de,(MZO.step.len)
add hl, de
ld (MZO.step.mcount),hl
lb17:
jp lb16brk
lb161:
getthis 0
;case 124
;v=0;
ld hl,0; const val
ld (MZO.step.v),hl
;mpoint++;
ld hl,(MZO.step.mpoint)
inc hl
ld (MZO.step.mpoint),hl
jp lb16brk
lb162:
getthis 0
;case 104
;mpoint++;
ld hl,(MZO.step.mpoint)
inc hl
ld (MZO.step.mpoint),hl
;var ofs=peekw(mpoint)
ld hl,(MZO.step.mpoint)
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
ld (MZO.step.ofs),hl
;mpoint--;
ld hl,(MZO.step.mpoint)
dec hl
ld (MZO.step.mpoint),hl
;mpoint+=ofs-65536;
ld hl,0; const val
push hl
ld hl,(MZO.step.ofs)
pop de
subhl de
push hl
ld hl,(MZO.step.mpoint)
pop de
add hl, de
ld (MZO.step.mpoint),hl
jp lb16brk
lb163:
getthis 0
;case 255
;stop(ch);
ld hl,(MZO.step.ch)
push hl
call MZO.stop
getthis 0
;mpoint=0;
ld hl,0; const val
ld (MZO.step.mpoint),hl
;mcount=1;
ld hl,1; const val
ld (MZO.step.mcount),hl
jp lb16brk
lb164:
getthis 0
;case 100
;mpoint+=2;
ld hl,(MZO.step.mpoint)
ld de,2; const val
add hl, de
ld (MZO.step.mpoint),hl
jp lb16brk
lb165:
getthis 0
;case 101
;mpoint+=2;
ld hl,(MZO.step.mpoint)
ld de,2; const val
add hl, de
ld (MZO.step.mpoint),hl
jp lb16brk
jp lb16brk
lb166:
getthis 0
;case 102
;mpoint+=2;
ld hl,(MZO.step.mpoint)
ld de,2; const val
add hl, de
ld (MZO.step.mpoint),hl
jp lb16brk
lb167:
getthis 0
;case 103
;tempoc=peekw(mpoint)*168/60;
ld hl,(MZO.step.mpoint)
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
ld de,168; const val
call mul.a
getthis 0
ld de,60; const val
call div.a
getthis 0
ld (MZO.step.tempoc),hl
;mpoint+=3;
ld hl,(MZO.step.mpoint)
ld de,3; const val
add hl, de
ld (MZO.step.mpoint),hl
jp lb16brk
lb168:
getthis 0
;case 107
;mpoint+=2;
ld hl,(MZO.step.mpoint)
ld de,2; const val
add hl, de
ld (MZO.step.mpoint),hl
jp lb16brk
lb169:
getthis 0
;case 110
;mpoint+=2;
ld hl,(MZO.step.mpoint)
ld de,2; const val
add hl, de
ld (MZO.step.mpoint),hl
jp lb16brk
lb1610:
getthis 0
;case 115
;mpoint+=4;
ld hl,(MZO.step.mpoint)
ld de,4; const val
add hl, de
ld (MZO.step.mpoint),hl
jp lb16brk
lb1611:
getthis 0
;case 116
;mpoint+=2;
ld hl,(MZO.step.mpoint)
ld de,2; const val
add hl, de
ld (MZO.step.mpoint),hl
jp lb16brk
lb1612:
getthis 0
;case 118
;mpoint+=2;
ld hl,(MZO.step.mpoint)
ld de,2; const val
add hl, de
ld (MZO.step.mpoint),hl
jp lb16brk
lb1613:
getthis 0
;case 120
;mpoint+=2;
ld hl,(MZO.step.mpoint)
ld de,2; const val
add hl, de
ld (MZO.step.mpoint),hl
jp lb16brk
lb1614:
getthis 0
;case 122
;mpoint++;
ld hl,(MZO.step.mpoint)
inc hl
ld (MZO.step.mpoint),hl
;len=peekw(mpoint);
ld hl,(MZO.step.mpoint)
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
ld (MZO.step.len),hl
;mpoint+=2;
ld hl,(MZO.step.mpoint)
ld de,2; const val
add hl, de
ld (MZO.step.mpoint),hl
jp lb16brk
lb16def:
;locate(0,6);
ld hl,6; const val
push hl
ld hl,0; const val
pop de
call locate
;printf ("Inv %d %x" ,cmd, mpoint);
#local
ld hl,st
jp nx
st: db 73, 110, 118, 32
db 0
nx:
#endlocal
call print.a
getthis 0
ld hl,(MZO.step.cmd)
call printi.a
getthis 0
#local
ld hl,st
jp nx
st: db 32
db 0
nx:
#endlocal
call print.a
getthis 0
ld hl,(MZO.step.mpoint)
show hl
getthis 0
;stop(ch);
ld hl,(MZO.step.ch)
push hl
call MZO.stop
getthis 0
;mpoint=0;
ld hl,0; const val
ld (MZO.step.mpoint),hl
;mcount=1;
ld hl,1; const val
ld (MZO.step.mcount),hl
jp lb16brk
lb16brk:
jp lb8
lb9:
;$mpoint[ch]=mpoint;
ld hl,(MZO.step.mpoint)
push hl
ld hl,(MZO.step.ch)
add hl,hl
ld de,MZOgbl_mpoint
add hl,de
pop de
ld (hl),e
inc hl
ld (hl),d
;$mcount[ch]=mcount;
ld hl,(MZO.step.mcount)
push hl
ld hl,(MZO.step.ch)
add hl,hl
ld de,MZOgbl_mcount
add hl,de
pop de
ld (hl),e
inc hl
ld (hl),d
;$tempoc[ch]=tempoc;
ld hl,(MZO.step.tempoc)
push hl
ld hl,(MZO.step.ch)
add hl,hl
ld de,MZOgbl_tempoc
add hl,de
pop de
ld (hl),e
inc hl
ld (hl),d
;$len[ch]=len;
ld hl,(MZO.step.len)
push hl
ld hl,(MZO.step.ch)
add hl,hl
ld de,MZOgbl_len
add hl,de
pop de
ld (hl),e
inc hl
ld (hl),d
;$v [ch]=v;
ld hl,(MZO.step.v)
push hl
ld hl,(MZO.step.ch)
add hl,hl
ld de,MZOgbl_v
add hl,de
pop de
ld (hl),e
inc hl
ld (hl),d
lbexit:
#endlocal
ret
MZO.initBGM:
pop bc
pop hl
ld (MZO.initBGM.mpoint),hl
push bc
#local
;"static all";
;mpoint+=4;
ld hl,(MZO.initBGM.mpoint)
ld de,4; const val
add hl, de
ld (MZO.initBGM.mpoint),hl
;mpoint++;
ld hl,(MZO.initBGM.mpoint)
inc hl
ld (MZO.initBGM.mpoint),hl
;var size
;size=peekw(mpoint);
ld hl,(MZO.initBGM.mpoint)
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
ld (MZO.initBGM.size),hl
;mpoint+=4;
ld hl,(MZO.initBGM.mpoint)
ld de,4; const val
add hl, de
ld (MZO.initBGM.mpoint),hl
;init(0,mpoint);
ld hl,(MZO.initBGM.mpoint)
push hl
ld hl,0; const val
push hl
call MZO.init
getthis 0
;mpoint+=size;
ld hl,(MZO.initBGM.mpoint)
ld de,(MZO.initBGM.size)
add hl, de
ld (MZO.initBGM.mpoint),hl
;size=peekw(mpoint);
ld hl,(MZO.initBGM.mpoint)
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
ld (MZO.initBGM.size),hl
;mpoint+=4;
ld hl,(MZO.initBGM.mpoint)
ld de,4; const val
add hl, de
ld (MZO.initBGM.mpoint),hl
;init(1,mpoint);
ld hl,(MZO.initBGM.mpoint)
push hl
ld hl,1; const val
push hl
call MZO.init
getthis 0
;mpoint+=size;
ld hl,(MZO.initBGM.mpoint)
ld de,(MZO.initBGM.size)
add hl, de
ld (MZO.initBGM.mpoint),hl
;size=peekw(mpoint);
ld hl,(MZO.initBGM.mpoint)
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
ld (MZO.initBGM.size),hl
;mpoint+=4;
ld hl,(MZO.initBGM.mpoint)
ld de,4; const val
add hl, de
ld (MZO.initBGM.mpoint),hl
;init(2,mpoint);
ld hl,(MZO.initBGM.mpoint)
push hl
ld hl,2; const val
push hl
call MZO.init
getthis 0
;mpoint+=size;
ld hl,(MZO.initBGM.mpoint)
ld de,(MZO.initBGM.size)
add hl, de
ld (MZO.initBGM.mpoint),hl
lbexit:
#endlocal
ret
MZO.initSE:
pop bc
pop hl
ld (MZO.initSE.mpoint),hl
push bc
#local
;"static all";
;if $mpoint [3] &&    $mpoint [3]<mpoint
ld hl,3; const val
add hl,hl
ld de,MZOgbl_mpoint; const val
add hl,de
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
andand lb20
ld hl,(MZO.initSE.mpoint)
push hl
ld hl,3; const val
add hl,hl
ld de,MZOgbl_mpoint; const val
add hl,de
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
pop de
call hlltde
lb20:
jpf lb19;then
;return return;
jp lbexit
lb19:
;mpoint+=4;
ld hl,(MZO.initSE.mpoint)
ld de,4; const val
add hl, de
ld (MZO.initSE.mpoint),hl
;mpoint++;
ld hl,(MZO.initSE.mpoint)
inc hl
ld (MZO.initSE.mpoint),hl
;var size=peekw(mpoint)
ld hl,(MZO.initSE.mpoint)
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
ld (MZO.initSE.size),hl
;mpoint+=4;
ld hl,(MZO.initSE.mpoint)
ld de,4; const val
add hl, de
ld (MZO.initSE.mpoint),hl
;init(3,mpoint);
ld hl,(MZO.initSE.mpoint)
push hl
ld hl,3; const val
push hl
call MZO.init
getthis 0
lbexit:
#endlocal
ret
MZO.stopBGM:
pop bc
push bc
#local
;"static all";
;for var i=0;
;var i=0
ld hl,0; const val
ld (MZO.stopBGM.i),hl
lb21:
;while i<3
ld hl,3; const val
push hl
ld hl,(MZO.stopBGM.i)
pop de
call hlltde
jpf lb22
;stop(i);
ld hl,(MZO.stopBGM.i)
push hl
call MZO.stop
getthis 0
lb23:
;next i++
ld hl,(MZO.stopBGM.i)
inc hl
ld (MZO.stopBGM.i),hl
jp lb21
lb22:
lbexit:
#endlocal
ret
MZO.stepAll:
pop bc
push bc
#local
;"static all";
;for var i=0;
;var i=0
ld hl,0; const val
ld (MZO.stepAll.i),hl
lb24:
;while i<4
ld hl,4; const val
push hl
ld hl,(MZO.stepAll.i)
pop de
call hlltde
jpf lb25
;step(i);
ld hl,(MZO.stepAll.i)
push hl
call MZO.step
getthis 0
lb26:
;next i++
ld hl,(MZO.stepAll.i)
inc hl
ld (MZO.stepAll.i),hl
jp lb24
lb25:
;$mute[2]=$mpoint [3];
ld hl,3; const val
add hl,hl
ld de,MZOgbl_mpoint; const val
add hl,de
ld e,(hl)
inc hl
ld d,(hl)
ex de,hl
push hl
ld hl,2
add hl,hl
ld de,MZOgbl_mute
add hl,de
pop de
ld (hl),e
inc hl
ld (hl),d
lbexit:
#endlocal
ret
;end of include z_mzo

;.onUpdate equ .c-2
;.update equ .onUpdate-2
;.screenOut equ .update-2
;.die equ .screenOut-2
;.updateEx equ .die-2

end.const macro n
 pushi RActor.wait,bc
 pushi o.boot,bc
 th.with.ret 0
 marker.e &n
endm

RActor.noovr macro Class
 ;keep'em. used in FldIdx
 ;fld .class,Class
 ;fld .statusReg,0
 meth &Class,main
 fld.bottom Object
 fld .x, 0
 fld .y, -1024
 fld .p, 0
 fld .c, 3
 fld.bottom Sprite
 meth RActor,onUpdate
 meth RActor,update
 meth RActor,screenOut
 meth RActor,die
 meth RActor,updateEx
 meth RActor,crashTo
 fld.bottom RActor
endm


crashToClass macro Class,st,en
#local
 ; a=this
 call crashTo.setXY
 foreach.a &Class,&st,&en,nx
  call crashTo1
  break.if c
  continue
 nx:
 getthis 0
 jr c, found
  ld hl,null
 found:
#endlocal
endm


foreach.a macro Class,st,en,nxt
 th.for.a &nxt, &st, &en
  all.skip &Class
endm


all.skip.blank.self macro xx
 ; skip blank
  ; TODO th.ofs.stp
  call th.isblank.a
  continue.if z
  ; skip hl==this
  getthis 0
  cp h
  continue.if z
endm
all.skip.isnot macro Class
  ; skip object not instance of *Class*
  push hl
  ld a,h
  ld de,&Class
  call instanceof
  getthis 0
  pop hl
  continue.if nz
endm
all.skip macro Class
 all.skip.blank.self 0
 all.skip.isnot &Class
endm


defsub onExitFrame
 ld hl,(gbl_frameCount)
 inc hl
 ld (gbl_frameCount),hl
 jp spr.puts
endsub onExitFrame

tnu.run macro Main,st,en
 unreach.new.init 0
 ld sp,sp.ini
 call screen1
 call screen2

 showsp
 showlb endusr
 call inipat

 ld hl,th.start
 ld (hl),0cdh
 ld de,th.start+1
 ld bc,th.size*th.count-1
 ldir

 call th.init
 
 ld hl, MZO.stepAll
 call setInterval
 
 ;ld a,0
 ;call MZOCmd
 ;call mus.ini
 new &Main, 0,&st ,&en
 ld hl,onExitFrame
 sethook h.thlop, hl
 ld hl,keyall
 sethook h.thent, hl
 jp th.loop
endm


loadPage macro Class,args,st,en
;call clearInterval
call all.die
new &Class,&args,&st,&en
endm



;end of include z_tnu
;include z_map




defsub map.adr
 ; hl=chipx
 ; de=chipy
 rept 5
  slde 0
 endm
 add hl,de
 ld de,1800h
 add hl,de
endsub map.adr

defsub map.set.a
 ;  a=data
 call map.adr
 call 4dh
endsub map.set.a

defsub map.get.a
 call map.adr
 call 4ah
endsub map.get.a

defsub map.adrat.a
 ; hl=spr_x
 ; de=spr_y
 ld bc,(gbl_offsetX)
 and a
 sbc hl,bc
 spr.unscale 0
 srl a
 srl a
 srl a
 push af
 ex de,hl
 ld bc,(gbl_offsetY)
 and a
 sbc hl,bc
 spr.unscale 0
 srl a
 srl a
 srl a
 ld d,0
 ld e,a
 pop hl
 ld l,h
 ld h,0
 call map.adr
endsub map.adrat.a

defsub map.getat.a
 call map.adrat.a
 call 4ah
endsub map.getat.a

defsub map.setat.a
 ; a=data
 push af
 call map.adrat.a
 pop af
 call 4dh
endsub map.setat.a

defsub locate
 ; hl=chipx
 ; de=chipy
 call map.adr
 ld (cursor),hl
endsub locate


;end of include z_map

;include z_dac

;jp main









DECSUB equ 268CH;DAC ← DAC-ARG
DECADD equ 269AH;DAC ← DAC+ARG
DECNRM equ 26FAH;DAC を正規化する (*1)
DECROU equ 273CH;DAC を四捨五入する
DECMUL equ 27E6H;DAC ← DAC*DAC
DECDIV equ 289FH;DAC ← DAC/DAC
MAF equ 2C4DH;ARG ← DAC
MAM equ 2C50H;ARG ← [HL]
MOV8DH equ 2C53H;[DE] ← [HL]
MFA equ 2C59H;DAC ← ARG
MFM equ 2C5CH;[HL] ← DAC
MMF equ 2C67H;[HL] ← DAC
MOV8HD equ 2C6AH;[HL] ← [DE]
XTF equ 2C6FH;[SP] ←→ DAC
PHA equ 2CC7H;ARG → [SP]
PHF equ 2CCCH;DAC → [SP]
PPA equ 2CDCH;[SP] → ARG
PPF equ 2CE1H;[SP] → DAC
PUSHF equ 2EB1H;DAC → [SP]
MOVFM equ 2EBEH;DAC ← [HL]
MOVFR equ 2EC1H;DAC ← (CBED)
MOVRF equ 2ECCH;(CBED) ← DAC
MOVRMI equ 2ED6H;(CBDE) ← [HL]
MOVRM equ 2EDFH;(BCDE) ← [HL]
MOVMF equ 2EE8H;[HL] ← DAC
MOVE equ 2EEBH;[HL] ← [DE]
VMOVAM equ 2EEFH;ARG ← [HL]
MOVVFM equ 2EF2H;[DE] ← [HL]
VMOVE equ 2EF3H;[HL] ← [DE]
VMOVFA equ 2F05H;DAC ← ARG
VMOVFM equ 2F08H;DAC ← [HL]
VMOVAF equ 2F0DH;ARG ← DAC
VMOVMF equ 2F10H;[HL] ← DAC

VALTYP equ 0F663H;1
DAC equ 0F7F6H;16
ARG equ 0F847H;16
FOUT equ 3425H
PUFOUT equ 3426H

defsub int2dac
 push af
 ld a,2
 ld (VALTYP),a
 ld (DAC+2),HL
 pop af
endsub int2dac
;===your code

#local
main:
ld hl,12345
call int2dac
ld hl,str
call FOUT

ld b,10
reptb nxt
 ld a,(hl)
 cp 0
 break.if z
 call wrt
 inc hl
 continue
nxt:
ret
str:
#endlocal
defsub myfout.a
#local
 xor a
 ld (strbuf.i+1),a
 bit 7,h
 jr z, posi
nega:
 neghl.sv 0
 ld a, '-'
 ld (strbuf.i),a
 jr loop
posi:
 ld a, space.char
 ld (strbuf.i),a
loop:
 ld de,10
 call udiv.a
 ;show bc
 ; bc=q hl=r
 ld a,l
 push bc
 call unshift.sbuf
 pop hl
 add a,'0'
 ld (strbuf.i+1),a
 ld a,h
 or l
 jr nz, loop
 
#endlocal
endsub myfout.a

defsub printi.a
 ;call int2dac
 call myfout.a
 ld hl,strbuf.i
 ;ld a,128
 ;ld b,0
 ;call FOUT
 call print.a
endsub printi.a
; 0123456
; s12345
;       
defsub unshift.sbuf
 ld hl, strbuf.i+5
 ld de, strbuf.i+6
 ld bc, 5
 lddr
endsub unshift.sbuf


;end of include z_dac
;include z_mus



defsub mus.scale.a
 ; a: 0,1,2 ch
 ; l: 0-95
 sla a
 sla l
 ld h,0
 ld bc,scltbl
 add hl,bc
 ld e,(hl)
 call WRTPSG
 inc hl
 ld e,(hl)
 inc a
 call WRTPSG
endsub mus.scale.a

deftbl scltbl
dw 0d5dh, 0c9ch, 0be7h, 0b3ch, 0a9bh, 0a02h, 0973h, 08ebh, 086bh, 07f2h, 0780h, 0714h, 06afh, 064eh, 05f4h, 059eh, 054eh, 0501h, 04bah, 0476h, 0436h, 03f9h, 03c0h, 038ah, 0357h, 0327h, 02fah, 02cfh, 02a7h, 0281h, 025dh, 023bh, 021bh, 01fdh, 01e0h, 01c5h, 01ach, 0194h, 017dh, 0168h, 0153h, 0140h, 012eh, 011dh, 010dh, 0feh, 0f0h, 0e3h, 0d6h, 0cah, 0beh, 0b4h, 0aah, 0a0h, 097h, 08fh, 087h, 07fh, 078h, 071h, 06bh, 065h, 05fh, 05ah, 055h, 050h, 04ch, 047h, 043h, 040h, 03ch, 039h, 035h, 032h, 030h, 02dh, 02ah, 028h, 026h, 024h, 022h, 020h, 01eh, 01ch, 01bh, 019h, 018h, 016h, 015h, 014h, 013h, 012h, 011h, 010h, 0fh, 0eh
endtbl scltbl
;end of include z_mus
;!ROM
gbl_se_bgms: 
db 220, 5, 0, 0, 10, 29, 0, 0, 0, 124, 101, 20, 120, 0, 122, 216
db 9, 50, 53, 57, 50, 53, 57, 60, 59, 57, 60, 59, 50, 53, 104, 238
db 255, 255, 255, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4, 0
db 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4, 0
db 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4, 0
db 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4, 0
db 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255
gbl_se_miss: 
db 220, 5, 0, 0, 10, 62, 0, 0, 0, 124, 122, 118, 2, 101, 0, 100
db 40, 24, 99, 35, 33, 31, 29, 28, 26, 24, 24, 99, 35, 33, 31, 29
db 28, 26, 24, 24, 99, 35, 33, 31, 29, 28, 26, 24, 99, 43, 41, 40
db 38, 36, 35, 33, 31, 29, 28, 26, 24, 23, 21, 19, 17, 16, 14, 12
db 11, 9, 7, 5, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255
gbl_se_laserPre: 
db 220, 5, 0, 0, 10, 53, 0, 0, 0, 124, 122, 118, 2, 101, 0, 100
db 40, 33, 21, 9, 35, 23, 11, 36, 24, 12, 38, 26, 14, 40, 28, 16
db 41, 29, 17, 43, 31, 19, 45, 33, 21, 47, 35, 23, 48, 36, 24, 50
db 38, 26, 52, 40, 28, 53, 41, 29, 55, 43, 31, 255, 255, 255, 4, 0
db 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4, 0
db 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4, 0
db 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4, 0
db 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4, 0
db 0, 0, 124, 255, 255, 255
gbl_se_b: 
db 220, 5, 0, 0, 10, 14, 0, 0, 0, 124, 122, 118, 2, 101, 0, 100
db 40, 59, 47, 35, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255
gbl_se_a: 
db 220, 5, 0, 0, 10, 14, 0, 0, 0, 124, 122, 118, 2, 101, 0, 100
db 40, 57, 45, 33, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255
gbl_se_g: 
db 220, 5, 0, 0, 10, 14, 0, 0, 0, 124, 122, 118, 2, 101, 0, 100
db 40, 55, 43, 31, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255
gbl_se_f: 
db 220, 5, 0, 0, 10, 14, 0, 0, 0, 124, 122, 118, 2, 101, 0, 100
db 40, 53, 41, 29, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255
gbl_se_laser: 
db 220, 5, 0, 0, 10, 60, 0, 0, 0, 124, 122, 118, 2, 100, 40, 57
db 45, 33, 44, 32, 21, 44, 32, 21, 44, 32, 21, 44, 32, 21, 44, 32
db 21, 44, 32, 21, 44, 32, 21, 44, 32, 21, 44, 32, 21, 44, 32, 21
db 44, 32, 21, 44, 32, 21, 44, 32, 21, 44, 32, 21, 44, 32, 21, 44
db 32, 21, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4, 0, 0
db 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4, 0, 0
db 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4, 0, 0
db 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4, 0, 0
db 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255
gbl_se_iopen: 
db 220, 5, 0, 0, 10, 29, 0, 0, 0, 124, 120, 0, 122, 216, 9, 101
db 5, 26, 123, 176, 19, 99, 26, 26, 26, 26, 123, 176, 19, 99, 104, 236
db 255, 255, 255, 255, 255, 255, 29, 0, 0, 0, 124, 120, 0, 122, 216, 9
db 101, 5, 14, 123, 176, 19, 99, 14, 14, 14, 14, 123, 176, 19, 99, 104
db 236, 255, 255, 255, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255, 4, 0, 0, 0, 124, 255, 255, 255, 4
db 0, 0, 0, 124, 255, 255, 255
;!/ROM

;begin of global variables
;end of global variables

endusr:
;include z_vram


inivrm macro n, dst
 ld de,&dst
 ld hl,&n
 ld bc,end.&n-&n
 call LDIRVM
endm

inipat:
 inivrm spr.pat, 3800h
 inivrm bg.gen, 0000h
 inivrm bg.gen, 0800h
 inivrm bg.gen, 1000h
 inivrm bg.col, 2000h
 inivrm bg.col, 2800h
 inivrm bg.col, 3000h
 ld hl,1800h
 ld a,32
 ld bc,300h
 call filvrm
 ret

        
;end of include z_vram
spr.pat:
db 00ch, 00eh, 00fh, 04fh, 03dh, 01dh, 07fh, 01bh, 00ch, 03fh, 07fh, 07fh, 06fh, 00fh, 006h, 00ch
db 018h, 038h, 0f8h, 0f9h, 0deh, 0dch, 07fh, 06ch, 098h, 0fch, 0feh, 0feh, 0f6h, 0f0h, 060h, 070h
db 00ch, 00eh, 00fh, 04fh, 03dh, 01dh, 07fh, 01bh, 00ch, 03fh, 07fh, 07fh, 0efh, 0efh, 006h, 006h
db 018h, 038h, 0f8h, 0f9h, 0deh, 0dch, 07fh, 06ch, 098h, 0fch, 0feh, 0feh, 0d4h, 078h, 0f0h, 000h
db 018h, 01ch, 01fh, 09fh, 07bh, 03bh, 0feh, 036h, 019h, 03fh, 07fh, 07fh, 02bh, 01eh, 00fh, 000h
db 030h, 070h, 0f0h, 0f2h, 0bch, 0b8h, 0feh, 0d8h, 030h, 0fch, 0feh, 0feh, 0f7h, 0f7h, 060h, 060h
db 02ch, 08eh, 00fh, 04bh, 03dh, 011h, 07fh, 01dh, 0cah, 0ffh, 07fh, 03fh, 015h, 01fh, 00eh, 000h
db 01ch, 039h, 0f8h, 0e9h, 0deh, 0c4h, 07fh, 05ch, 0abh, 0ffh, 0ffh, 0feh, 0ach, 0f8h, 070h, 000h
db 0b8h, 05eh, 02fh, 017h, 00fh, 003h, 00fh, 01fh, 01fh, 03fh, 01fh, 003h, 00bh, 017h, 02eh, 0dch
db 000h, 000h, 000h, 000h, 080h, 0f0h, 0c8h, 0c4h, 0e2h, 0ffh, 0ffh, 0fch, 080h, 000h, 000h, 000h
db 008h, 010h, 030h, 030h, 018h, 00fh, 03fh, 037h, 07bh, 07dh, 03fh, 01fh, 037h, 060h, 070h, 00ch
db 008h, 004h, 006h, 006h, 00ch, 0f8h, 0feh, 0f6h, 0efh, 0dfh, 0feh, 0fch, 0f6h, 003h, 007h, 018h
db 000h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 000h, 000h
db 080h, 0c0h, 0c0h, 0c0h, 0c0h, 0c0h, 0c0h, 0c0h, 0c0h, 0c0h, 0c0h, 0c0h, 0c0h, 0c0h, 080h, 000h
db 000h, 000h, 000h, 000h, 000h, 001h, 003h, 001h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 080h, 0c0h, 080h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 003h, 00fh, 00fh, 01fh, 01eh, 01eh, 01fh, 00fh, 00fh, 003h, 000h, 000h, 000h
db 000h, 000h, 000h, 0c0h, 0f0h, 0f0h, 0f8h, 078h, 078h, 0f8h, 0f0h, 0f0h, 0c0h, 000h, 000h, 000h
db 000h, 000h, 007h, 00fh, 018h, 030h, 030h, 030h, 030h, 030h, 018h, 00fh, 007h, 000h, 000h, 000h
db 000h, 000h, 0c0h, 0e0h, 030h, 018h, 018h, 018h, 018h, 018h, 030h, 0e0h, 0c0h, 000h, 000h, 000h
db 000h, 003h, 007h, 00eh, 01ch, 038h, 030h, 030h, 030h, 038h, 01ch, 00eh, 007h, 003h, 000h, 000h
db 000h, 0e0h, 0f0h, 038h, 01ch, 00eh, 006h, 006h, 006h, 00eh, 01ch, 038h, 0f0h, 0e0h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 003h, 047h, 02fh, 01dh, 01dh, 03fh, 05fh, 01ch, 00fh, 007h, 003h, 004h, 004h, 004h, 000h
db 000h, 0c0h, 0e2h, 0f4h, 0b8h, 0b8h, 0fch, 0fah, 038h, 0f0h, 0e0h, 0c0h, 020h, 020h, 020h, 000h
db 080h, 040h, 02fh, 01fh, 03fh, 03bh, 03bh, 03fh, 03fh, 03fh, 03fh, 030h, 01fh, 02fh, 040h, 080h
db 001h, 002h, 0f4h, 0f8h, 0fch, 0dch, 0dch, 0fch, 0fch, 0fch, 0fch, 00ch, 0f8h, 0f4h, 002h, 001h
db 000h, 007h, 01fh, 03bh, 03bh, 03fh, 03fh, 038h, 01fh, 00fh, 012h, 0e2h, 042h, 002h, 004h, 00ch
db 000h, 0e0h, 0f8h, 0dch, 0dch, 0fch, 0fch, 01ch, 0f8h, 0f0h, 048h, 047h, 042h, 040h, 020h, 030h
db 000h, 007h, 007h, 00fh, 00dh, 00dh, 00fh, 00fh, 00fh, 00ch, 01fh, 03fh, 077h, 067h, 046h, 000h
db 000h, 0e0h, 0e0h, 0f0h, 0b0h, 0b0h, 0f0h, 0f0h, 0f0h, 030h, 0f8h, 0fch, 0eeh, 0e6h, 062h, 000h
db 03fh, 03fh, 0edh, 0edh, 0ffh, 0c7h, 07fh, 00fh, 01fh, 03fh, 03eh, 07fh, 07fh, 07fh, 03fh, 00fh
db 0e0h, 0e0h, 0e0h, 0e0h, 0c0h, 0c0h, 0c0h, 080h, 000h, 000h, 007h, 00fh, 03fh, 0feh, 0fch, 0f0h
db 000h, 000h, 007h, 00fh, 01fh, 03fh, 07fh, 07fh, 0ffh, 0ffh, 0ffh, 0feh, 0ffh, 07fh, 07eh, 000h
db 00fh, 0ffh, 0ffh, 0ffh, 0ffh, 0b7h, 0b7h, 0ffh, 0feh, 0feh, 0fch, 000h, 0f0h, 0c0h, 000h, 000h
db 00fh, 03fh, 07fh, 07bh, 0fbh, 0ffh, 0ffh, 078h, 07fh, 03fh, 03fh, 01ah, 032h, 062h, 07ah, 008h
db 0f0h, 0fch, 0feh, 0deh, 0dfh, 0ffh, 0ffh, 01eh, 0feh, 0fch, 0fch, 058h, 04ch, 046h, 05eh, 010h
db 020h, 030h, 03dh, 03fh, 03fh, 01fh, 0dfh, 0fdh, 07dh, 07fh, 07fh, 03fh, 03ch, 01fh, 007h, 003h
db 004h, 00ch, 0bch, 0fch, 0fch, 0f8h, 0fbh, 0bfh, 0beh, 0feh, 0feh, 0fch, 03ch, 0f8h, 0e0h, 0c0h
db 001h, 001h, 003h, 003h, 007h, 00fh, 00fh, 01bh, 03bh, 03fh, 03fh, 03fh, 03ch, 01fh, 00fh, 003h
db 080h, 080h, 0c0h, 0c0h, 0e0h, 0f0h, 0f0h, 0d8h, 0dch, 0fch, 0fch, 0fch, 03ch, 0f8h, 0f0h, 0c0h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
end.spr.pat:
bg.gen:
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 00fh, 00fh, 00fh, 00fh, 00fh, 00fh, 00fh, 00fh
db 000h, 080h, 080h, 080h, 080h, 080h, 080h, 000h, 000h, 080h, 080h, 080h, 080h, 080h, 080h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 00fh, 0e0h, 0c0h, 083h, 0f0h
db 000h, 000h, 000h, 0c0h, 0f0h, 007h, 003h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 001h, 001h, 003h, 003h, 005h, 00dh, 00eh, 01eh
db 0c0h, 0c0h, 0e0h, 0f0h, 0f0h, 007h, 007h, 0d4h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 001h, 007h, 00fh, 0e0h, 0c0h, 080h, 0f0h, 0c0h, 0e0h, 0c0h, 0c0h, 080h, 000h, 000h, 000h, 000h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 080h, 0e0h, 0f0h, 007h, 003h, 000h, 002h, 005h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 0c0h, 0c0h, 0b0h, 090h, 088h, 00dh, 002h, 000h
db 02ch, 011h, 001h, 023h, 02bh, 048h, 060h, 022h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 080h, 000h, 000h, 000h, 000h, 000h, 038h, 000h, 000h, 000h, 002h, 005h, 00ah, 0c0h, 0c0h, 0c0h
db 00ah, 015h, 055h, 0aah, 055h, 007h, 003h, 0c1h, 02ah, 0a8h, 055h, 0aah, 054h, 000h, 000h, 000h
db 001h, 003h, 003h, 003h, 007h, 007h, 007h, 007h, 000h, 000h, 000h, 080h, 041h, 022h, 014h, 008h
db 028h, 024h, 043h, 083h, 002h, 082h, 040h, 031h, 080h, 0c0h, 080h, 020h, 0c0h, 0a0h, 040h, 090h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 080h, 080h, 080h, 080h, 080h, 080h, 083h, 0c0h
db 001h, 001h, 0e0h, 0e0h, 0c0h, 0c0h, 001h, 001h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 009h, 00eh, 0e0h, 0e0h, 001h, 01ch, 0c0h, 080h, 000h, 000h, 083h, 0a3h, 030h, 00fh, 0c0h, 000h
db 00ah, 046h, 085h, 002h, 005h, 003h, 000h, 000h, 0a0h, 058h, 0a8h, 058h, 080h, 01ch, 083h, 001h
db 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 0c0h, 0c0h, 0c0h, 01fh, 00fh, 00fh, 007h, 003h
db 001h, 003h, 007h, 007h, 0f0h, 0e0h, 0e0h, 0c0h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
db 080h, 000h, 000h, 000h, 000h, 080h, 0c0h, 007h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 007h
db 000h, 000h, 000h, 000h, 000h, 000h, 0f0h, 000h, 001h, 003h, 003h, 003h, 007h, 0c0h, 001h, 0c0h
db 083h, 0c6h, 031h, 029h, 019h, 0c6h, 083h, 000h, 018h, 038h, 078h, 018h, 018h, 018h, 081h, 000h
db 083h, 0c6h, 006h, 01ch, 070h, 0c2h, 001h, 000h, 083h, 0c6h, 006h, 03ch, 006h, 0c6h, 083h, 000h
db 01ch, 03ch, 06ch, 0cch, 001h, 00ch, 00ch, 000h, 001h, 0c0h, 003h, 006h, 006h, 0c6h, 083h, 000h
db 083h, 0c6h, 0c0h, 003h, 0c6h, 0c6h, 083h, 000h, 001h, 0c6h, 00ch, 01ch, 018h, 038h, 038h, 000h
db 083h, 0c6h, 0c6h, 083h, 0c6h, 0c6h, 083h, 000h, 083h, 0c6h, 0c6h, 081h, 006h, 004h, 078h, 000h
db 000h, 030h, 030h, 000h, 000h, 030h, 030h, 000h, 000h, 030h, 030h, 000h, 000h, 030h, 030h, 060h
db 010h, 030h, 081h, 001h, 081h, 030h, 010h, 000h, 000h, 000h, 083h, 000h, 083h, 000h, 000h, 000h
db 010h, 018h, 003h, 001h, 003h, 018h, 010h, 000h, 03ch, 066h, 006h, 00ch, 018h, 000h, 018h, 000h
db 083h, 082h, 00dh, 08ah, 045h, 0cah, 083h, 000h, 01ch, 036h, 036h, 066h, 081h, 0c6h, 0c6h, 000h
db 003h, 066h, 066h, 083h, 066h, 066h, 003h, 000h, 083h, 019h, 0c6h, 0c0h, 0c0h, 019h, 083h, 000h
db 003h, 066h, 066h, 066h, 066h, 066h, 003h, 000h, 001h, 066h, 060h, 083h, 060h, 066h, 001h, 000h
db 001h, 066h, 060h, 083h, 060h, 060h, 0f0h, 000h, 083h, 019h, 0c0h, 031h, 0c6h, 019h, 081h, 000h
db 0c6h, 0c6h, 0c6h, 001h, 0c6h, 0c6h, 0c6h, 000h, 078h, 030h, 030h, 030h, 030h, 030h, 078h, 000h
db 0c1h, 00ch, 00ch, 00ch, 0cch, 0cch, 078h, 000h, 019h, 06ch, 078h, 070h, 068h, 06ch, 019h, 000h
db 0f0h, 060h, 060h, 060h, 060h, 062h, 001h, 000h, 0c6h, 011h, 001h, 029h, 029h, 0c6h, 0c6h, 000h
db 0c6h, 019h, 019h, 029h, 031h, 031h, 0c6h, 000h, 083h, 0c6h, 0c6h, 0c6h, 0c6h, 0c6h, 083h, 000h
db 003h, 066h, 066h, 066h, 083h, 060h, 0f0h, 000h, 083h, 0c6h, 0c6h, 0c6h, 009h, 031h, 080h, 000h
db 003h, 066h, 066h, 066h, 083h, 06ch, 019h, 000h, 083h, 0c6h, 0c0h, 083h, 006h, 0c6h, 083h, 000h
db 001h, 025h, 018h, 018h, 018h, 018h, 03ch, 000h, 0c6h, 0c6h, 0c6h, 0c6h, 0c6h, 0c6h, 083h, 000h
db 0c6h, 0c6h, 0c6h, 0c6h, 06ch, 083h, 038h, 000h, 0c6h, 0c6h, 029h, 029h, 001h, 011h, 0c6h, 000h
db 0c6h, 0c6h, 06ch, 038h, 06ch, 0c6h, 0c6h, 000h, 0c6h, 0c6h, 066h, 03ch, 018h, 018h, 018h, 000h
db 001h, 0cch, 018h, 030h, 062h, 0c6h, 001h, 000h, 078h, 060h, 060h, 060h, 060h, 060h, 078h, 000h
db 0c6h, 06ch, 038h, 083h, 030h, 083h, 030h, 000h, 078h, 018h, 018h, 018h, 018h, 018h, 078h, 000h
db 010h, 038h, 064h, 0c2h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 083h, 000h
db 060h, 060h, 030h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 078h, 00ch, 083h, 0cch, 081h, 000h
db 080h, 0c0h, 007h, 0cch, 0cch, 0cch, 007h, 000h, 000h, 000h, 078h, 0cch, 0c0h, 0cch, 078h, 000h
db 00ch, 00ch, 083h, 0cch, 0cch, 0cch, 074h, 000h, 000h, 000h, 078h, 0cch, 003h, 0c0h, 083h, 000h
db 010h, 038h, 030h, 003h, 030h, 030h, 030h, 060h, 000h, 000h, 074h, 0cch, 0cch, 083h, 00ch, 078h
db 0c0h, 0c0h, 007h, 0cch, 0cch, 0cch, 0cch, 000h, 030h, 000h, 070h, 030h, 030h, 030h, 078h, 000h
db 018h, 000h, 038h, 018h, 018h, 018h, 0d8h, 070h, 060h, 060h, 06ch, 078h, 070h, 078h, 06ch, 000h
db 070h, 030h, 030h, 030h, 030h, 030h, 078h, 000h, 000h, 000h, 013h, 001h, 029h, 029h, 029h, 000h
db 000h, 000h, 0b8h, 0cch, 0cch, 0cch, 0cch, 000h, 000h, 000h, 078h, 0cch, 0cch, 0cch, 078h, 000h
db 000h, 000h, 0b8h, 0cch, 0cch, 007h, 0c0h, 0c0h, 000h, 000h, 074h, 0cch, 0cch, 083h, 00ch, 00ch
db 000h, 000h, 0d8h, 013h, 0c0h, 0c0h, 0c0h, 000h, 000h, 000h, 083h, 0c0h, 007h, 00ch, 007h, 000h
db 060h, 060h, 0f0h, 060h, 060h, 068h, 030h, 000h, 000h, 000h, 0d8h, 0d8h, 0d8h, 0d8h, 06ch, 000h
db 000h, 000h, 0c8h, 0c4h, 0c4h, 048h, 030h, 000h, 000h, 000h, 0c8h, 0c4h, 0d4h, 0d4h, 068h, 000h
db 000h, 000h, 0c4h, 068h, 030h, 068h, 0c4h, 000h, 000h, 000h, 0c4h, 0c4h, 0cch, 074h, 004h, 078h
db 000h, 000h, 003h, 018h, 030h, 060h, 003h, 000h, 01ch, 030h, 030h, 060h, 030h, 030h, 01ch, 000h
db 030h, 030h, 030h, 000h, 030h, 030h, 030h, 000h, 0e0h, 030h, 030h, 018h, 030h, 030h, 0e0h, 000h
db 000h, 000h, 070h, 09ah, 00ch, 000h, 000h, 000h, 010h, 030h, 081h, 001h, 081h, 030h, 010h, 000h
db 010h, 038h, 083h, 001h, 001h, 038h, 083h, 000h, 06ch, 001h, 001h, 001h, 083h, 038h, 010h, 000h
db 038h, 038h, 001h, 001h, 029h, 010h, 083h, 000h, 010h, 038h, 083h, 001h, 083h, 038h, 010h, 000h
db 000h, 078h, 084h, 084h, 084h, 084h, 078h, 000h, 000h, 078h, 003h, 003h, 003h, 003h, 078h, 000h
db 020h, 007h, 026h, 03ch, 044h, 040h, 0c1h, 000h, 000h, 000h, 010h, 083h, 03ch, 05ah, 034h, 000h
db 000h, 000h, 000h, 044h, 042h, 052h, 020h, 000h, 000h, 000h, 03ch, 0c1h, 042h, 002h, 01ch, 000h
db 000h, 000h, 018h, 081h, 004h, 018h, 091h, 000h, 000h, 000h, 012h, 081h, 03ch, 052h, 034h, 000h
db 000h, 000h, 048h, 003h, 04ah, 042h, 024h, 000h, 000h, 000h, 008h, 043h, 0cah, 00ch, 038h, 000h
db 000h, 000h, 008h, 00eh, 038h, 04ch, 03ah, 000h, 000h, 000h, 000h, 083h, 002h, 002h, 01ch, 000h
db 038h, 044h, 08eh, 092h, 0e2h, 044h, 038h, 000h, 020h, 003h, 028h, 03ch, 06ah, 092h, 064h, 000h
db 000h, 044h, 082h, 082h, 082h, 090h, 060h, 000h, 038h, 000h, 083h, 082h, 002h, 004h, 038h, 000h
db 038h, 000h, 041h, 044h, 008h, 038h, 031h, 000h, 020h, 085h, 022h, 03ch, 062h, 0a2h, 06ch, 000h
db 020h, 00bh, 02ah, 04ah, 048h, 088h, 030h, 000h, 010h, 083h, 008h, 0c1h, 084h, 080h, 083h, 000h
db 00ch, 018h, 020h, 040h, 020h, 018h, 00ch, 000h, 088h, 084h, 041h, 084h, 084h, 084h, 048h, 000h
db 000h, 083h, 002h, 004h, 040h, 040h, 0c1h, 000h, 020h, 016h, 007h, 008h, 044h, 080h, 083h, 000h
db 080h, 040h, 040h, 040h, 042h, 044h, 038h, 000h, 008h, 001h, 008h, 038h, 048h, 038h, 008h, 010h
db 004h, 044h, 001h, 044h, 04ch, 040h, 0c1h, 000h, 083h, 018h, 0c1h, 0d0h, 020h, 020h, 01ch, 000h
db 022h, 098h, 02dh, 062h, 046h, 0b4h, 019h, 044h, 000h, 000h, 000h, 000h, 060h, 090h, 060h, 000h
db 03ch, 030h, 030h, 030h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 018h, 018h, 018h, 078h, 000h
db 000h, 000h, 000h, 000h, 0c0h, 060h, 060h, 000h, 000h, 000h, 000h, 030h, 030h, 000h, 000h, 000h
db 001h, 002h, 001h, 002h, 004h, 018h, 060h, 000h, 000h, 000h, 007h, 008h, 030h, 020h, 040h, 000h
db 000h, 000h, 00ch, 030h, 0d0h, 010h, 010h, 000h, 000h, 020h, 007h, 088h, 008h, 010h, 060h, 000h
db 000h, 000h, 007h, 020h, 020h, 020h, 007h, 000h, 000h, 010h, 007h, 030h, 050h, 090h, 010h, 000h
db 000h, 040h, 003h, 044h, 048h, 040h, 020h, 000h, 000h, 000h, 078h, 008h, 008h, 008h, 003h, 000h
db 000h, 000h, 007h, 008h, 007h, 008h, 007h, 000h, 000h, 000h, 0a4h, 0a4h, 004h, 008h, 070h, 000h
db 000h, 000h, 080h, 081h, 000h, 000h, 000h, 000h, 001h, 002h, 014h, 018h, 010h, 020h, 040h, 000h
db 006h, 008h, 018h, 068h, 008h, 008h, 008h, 000h, 010h, 001h, 082h, 082h, 004h, 008h, 030h, 000h
db 000h, 001h, 010h, 010h, 010h, 010h, 001h, 000h, 004h, 001h, 00ch, 00ch, 014h, 024h, 0c4h, 000h
db 010h, 081h, 012h, 022h, 022h, 042h, 046h, 000h, 010h, 081h, 010h, 001h, 010h, 010h, 010h, 000h
db 0c1h, 022h, 042h, 002h, 004h, 004h, 008h, 000h, 040h, 081h, 048h, 088h, 008h, 010h, 020h, 000h
db 000h, 081h, 002h, 002h, 002h, 002h, 081h, 000h, 044h, 001h, 044h, 044h, 004h, 008h, 030h, 000h
db 000h, 072h, 002h, 072h, 002h, 004h, 007h, 000h, 000h, 001h, 002h, 004h, 008h, 034h, 0c2h, 000h
db 040h, 001h, 044h, 044h, 048h, 040h, 0c1h, 000h, 042h, 042h, 022h, 004h, 004h, 018h, 060h, 000h
db 0c1h, 022h, 0a1h, 042h, 004h, 004h, 018h, 000h, 002h, 083h, 008h, 001h, 008h, 010h, 060h, 000h
db 052h, 052h, 052h, 004h, 004h, 008h, 070h, 000h, 083h, 000h, 001h, 008h, 008h, 010h, 060h, 000h
db 020h, 020h, 020h, 038h, 026h, 020h, 020h, 000h, 008h, 081h, 008h, 008h, 008h, 008h, 030h, 000h
db 000h, 03ch, 000h, 000h, 000h, 000h, 081h, 000h, 000h, 081h, 002h, 081h, 004h, 018h, 019h, 000h
db 010h, 001h, 004h, 018h, 074h, 092h, 010h, 000h, 002h, 002h, 004h, 004h, 008h, 030h, 0c0h, 000h
db 010h, 008h, 044h, 044h, 044h, 044h, 082h, 000h, 040h, 042h, 083h, 040h, 040h, 042h, 03ch, 000h
db 001h, 002h, 002h, 002h, 004h, 018h, 060h, 000h, 000h, 020h, 050h, 088h, 084h, 002h, 000h, 000h
db 010h, 001h, 010h, 054h, 052h, 092h, 010h, 000h, 000h, 001h, 002h, 004h, 028h, 010h, 00ch, 000h
db 003h, 000h, 078h, 000h, 000h, 003h, 002h, 000h, 010h, 020h, 040h, 040h, 084h, 082h, 001h, 002h
db 002h, 002h, 032h, 00ch, 004h, 01ah, 0e2h, 000h, 083h, 020h, 001h, 020h, 020h, 020h, 01eh, 000h
db 040h, 001h, 042h, 044h, 058h, 040h, 040h, 000h, 000h, 083h, 004h, 004h, 004h, 004h, 001h, 000h
db 000h, 081h, 002h, 081h, 002h, 002h, 081h, 000h, 083h, 000h, 001h, 002h, 002h, 004h, 038h, 000h
db 044h, 044h, 044h, 044h, 004h, 008h, 030h, 000h, 010h, 050h, 050h, 052h, 052h, 054h, 098h, 000h
db 080h, 080h, 080h, 082h, 084h, 088h, 0f0h, 000h, 000h, 001h, 082h, 082h, 082h, 082h, 001h, 000h
db 001h, 082h, 082h, 002h, 002h, 004h, 038h, 000h, 000h, 0e0h, 012h, 002h, 002h, 004h, 007h, 000h
db 090h, 048h, 000h, 000h, 000h, 000h, 000h, 000h, 060h, 090h, 060h, 000h, 000h, 000h, 000h, 000h
db 020h, 001h, 040h, 05ch, 082h, 0a0h, 061h, 000h, 020h, 001h, 040h, 083h, 082h, 002h, 03ch, 000h
db 000h, 043h, 042h, 002h, 002h, 004h, 038h, 000h, 000h, 003h, 010h, 020h, 040h, 040h, 038h, 000h
db 020h, 016h, 018h, 020h, 040h, 040h, 0c1h, 000h, 024h, 005h, 044h, 044h, 09ch, 0a6h, 018h, 000h
db 09ch, 0a2h, 084h, 080h, 080h, 0a0h, 0a1h, 000h, 008h, 05ch, 06ah, 052h, 049h, 0aah, 04ch, 000h
db 040h, 013h, 052h, 062h, 031h, 04ah, 04ch, 000h, 000h, 03ch, 052h, 092h, 0a2h, 0a2h, 044h, 000h
db 004h, 041h, 084h, 084h, 09ch, 0a6h, 05ch, 000h, 008h, 04ch, 0c6h, 044h, 044h, 048h, 030h, 000h
db 020h, 018h, 020h, 014h, 08ah, 0cah, 030h, 000h, 000h, 000h, 030h, 068h, 0c4h, 006h, 002h, 000h
db 041h, 084h, 041h, 084h, 09ch, 0a6h, 05ah, 000h, 008h, 081h, 008h, 081h, 038h, 04ch, 032h, 000h
db 010h, 074h, 014h, 081h, 0a4h, 0a4h, 048h, 000h, 020h, 007h, 022h, 062h, 0a0h, 062h, 01ch, 000h
db 048h, 05ch, 06ah, 0cah, 0b2h, 0a2h, 054h, 000h, 040h, 040h, 007h, 040h, 005h, 042h, 03ch, 000h
db 040h, 048h, 083h, 0cah, 042h, 04ch, 040h, 020h, 008h, 043h, 04ah, 08ah, 043h, 008h, 030h, 000h
db 008h, 008h, 00eh, 008h, 078h, 08ch, 072h, 000h, 038h, 084h, 080h, 043h, 0c2h, 002h, 03ch, 000h
db 004h, 042h, 042h, 042h, 062h, 004h, 018h, 000h, 083h, 008h, 030h, 023h, 062h, 092h, 083h, 000h
db 024h, 02ch, 00bh, 024h, 064h, 0a4h, 022h, 000h, 083h, 008h, 01ch, 062h, 082h, 004h, 018h, 000h
db 040h, 060h, 023h, 062h, 042h, 0c2h, 05ch, 000h, 020h, 020h, 040h, 070h, 0c8h, 08ah, 084h, 000h
db 0aah, 055h, 0aah, 055h, 0aah, 055h, 0aah, 055h, 000h, 000h, 000h, 000h, 000h, 000h, 000h, 000h
end.bg.gen:
bg.col:
db 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h
db 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 081h, 091h, 0a2h, 0b3h, 0c4h, 0d5h, 0e6h, 0f7h
db 001h, 01ah, 01bh, 01ah, 01ah, 019h, 01ah, 001h, 001h, 012h, 013h, 012h, 012h, 01ch, 012h, 001h
db 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h
db 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 091h, 019h, 019h, 069h, 096h
db 001h, 001h, 001h, 061h, 061h, 016h, 016h, 006h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h
db 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 091h, 081h, 061h, 061h, 061h, 061h, 061h, 091h
db 091h, 081h, 061h, 061h, 061h, 016h, 016h, 061h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h
db 091h, 091h, 091h, 019h, 019h, 019h, 096h, 096h, 096h, 096h, 096h, 096h, 006h, 006h, 006h, 006h
db 006h, 006h, 006h, 006h, 006h, 006h, 006h, 006h, 061h, 061h, 061h, 016h, 016h, 006h, 016h, 016h
db 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 016h, 016h, 016h, 016h, 016h, 019h, 016h, 006h
db 061h, 016h, 016h, 016h, 061h, 016h, 016h, 016h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h
db 096h, 006h, 006h, 006h, 006h, 006h, 061h, 001h, 006h, 006h, 016h, 016h, 016h, 06ah, 01ah, 01bh
db 016h, 016h, 061h, 061h, 061h, 01ah, 01ah, 0abh, 016h, 061h, 061h, 061h, 061h, 001h, 001h, 001h
db 061h, 061h, 091h, 061h, 061h, 061h, 091h, 061h, 006h, 006h, 006h, 016h, 016h, 016h, 016h, 016h
db 016h, 016h, 016h, 016h, 016h, 016h, 016h, 016h, 061h, 061h, 061h, 061h, 061h, 061h, 061h, 061h
db 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 01bh, 01bh, 01bh, 01bh, 01bh, 01bh, 0abh, 01bh
db 01bh, 01ah, 0bah, 0bah, 0bah, 0bah, 01ah, 01ah, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h
db 061h, 061h, 016h, 016h, 061h, 0b1h, 01bh, 01bh, 006h, 006h, 016h, 061h, 016h, 061h, 0b1h, 00bh
db 016h, 016h, 016h, 016h, 016h, 016h, 001h, 00bh, 061h, 061h, 061h, 061h, 061h, 0b1h, 01bh, 01bh
db 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 01bh, 01bh, 01bh, 0b1h, 0b1h, 0b1h, 0b1h, 0b1h
db 01ah, 01ah, 01ah, 01ah, 0a1h, 0a1h, 0a1h, 0a1h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h
db 01bh, 00bh, 00bh, 00bh, 00bh, 01bh, 01bh, 0b1h, 00bh, 00bh, 00bh, 00bh, 00bh, 00bh, 00bh, 0abh
db 00bh, 00bh, 00bh, 00bh, 00bh, 00bh, 0bah, 00ah, 01bh, 0abh, 0abh, 0abh, 0abh, 0bah, 01ah, 0a1h
db 01fh, 0f1h, 01fh, 01fh, 01fh, 0f1h, 01fh, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h
db 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 001h, 01fh, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 01fh, 001h
db 01fh, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 01fh, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 01fh, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 01fh, 001h, 01fh, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h
db 001h, 0f1h, 0f1h, 001h, 001h, 0f1h, 0f1h, 001h, 001h, 0f1h, 0f1h, 001h, 001h, 0f1h, 0f1h, 0f1h
db 0f1h, 0f1h, 01fh, 01fh, 01fh, 0f1h, 0f1h, 001h, 001h, 001h, 01fh, 001h, 01fh, 001h, 001h, 001h
db 0f1h, 0f1h, 01fh, 01fh, 01fh, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 001h
db 01fh, 0f1h, 01fh, 0f1h, 01fh, 0f1h, 01fh, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 001h
db 01fh, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 01fh, 001h, 01fh, 01fh, 0f1h, 0f1h, 0f1h, 01fh, 01fh, 001h
db 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h, 01fh, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 01fh, 001h
db 01fh, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h, 01fh, 01fh, 0f1h, 01fh, 0f1h, 01fh, 01fh, 001h
db 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h, 0f1h, 01fh, 01fh, 01fh, 01fh, 0f1h, 0f1h, 001h
db 0f1h, 01fh, 01fh, 01fh, 01fh, 01fh, 0f1h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h
db 01fh, 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 01fh, 01fh, 01fh, 001h
db 01fh, 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 01fh, 001h, 01fh, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 01fh, 001h
db 01fh, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 001h, 0f1h, 0f1h, 01fh, 01fh, 01fh, 01fh, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 01fh, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 01fh, 001h
db 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 0f1h, 0f1h, 01fh, 0f1h, 01fh, 001h
db 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 01fh, 001h, 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 0f1h, 0f1h, 01fh, 0f1h, 01fh, 001h
db 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h
db 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 01fh, 01fh, 01fh, 01fh, 01fh, 001h
db 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 001h, 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 001h, 001h, 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h
db 001h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 01fh, 0f1h, 01fh, 0f1h, 01fh, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h
db 001h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 01fh, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 001h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 0f1h, 0f1h, 01fh, 01fh, 01fh, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 01fh, 01fh, 01fh, 0f1h, 01fh, 001h, 0f1h, 01fh, 01fh, 01fh, 01fh, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 01fh, 01fh, 01fh, 0f1h, 01fh, 001h, 0f1h, 0f1h, 01fh, 01fh, 01fh, 0f1h, 0f1h, 001h
db 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 0f1h, 01fh, 01fh, 01fh, 01fh, 0f1h, 001h
db 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h, 001h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h
db 001h, 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h
db 001h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 01fh, 001h, 001h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h
db 001h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h
db 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 01fh, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 01fh, 0f1h, 01fh, 0f1h, 0f1h, 01fh, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 01fh, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h
db 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 01fh, 001h, 01fh, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 001h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 001h, 001h, 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 001h, 001h, 001h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 001h, 0f1h, 0f1h, 001h, 001h, 001h
db 01fh, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 01fh, 001h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h
db 001h, 001h, 01fh, 0f1h, 01fh, 0f1h, 01fh, 001h, 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 001h, 0f1h, 01fh, 001h, 001h, 001h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 01fh, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h
db 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 01fh, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 01fh, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 01fh, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 001h, 0f1h, 001h, 001h, 001h, 001h, 01fh, 001h, 001h, 01fh, 0f1h, 01fh, 0f1h, 0f1h, 01fh, 001h
db 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h
db 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 01fh, 001h, 0f1h, 001h, 001h, 01fh, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 0f1h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 01fh, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h
db 001h, 01fh, 0f1h, 01fh, 0f1h, 0f1h, 01fh, 001h, 01fh, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h
db 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h
db 0f1h, 0f1h, 001h, 001h, 001h, 001h, 001h, 001h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 001h, 001h
db 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h, 0f1h, 01fh, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h
db 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 001h
db 0f1h, 01fh, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 001h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 01fh, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 01fh, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 01fh, 0f1h, 01fh, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 01fh, 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 01fh, 001h
db 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 01fh, 0f1h, 0f1h, 0f1h, 0f1h, 001h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 001h
db 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 0f1h, 00fh, 00fh, 00fh, 00fh, 00fh, 00fh, 00fh, 00fh
end.bg.col:


;include z_ram
 
if $<8000h
 ds 8000h-$, 0
endif
beginram macro
;--begin RAM area
endm
endram macro
;--end RAM area
endm
defwork macro n
 head &n
endm
endwork macro n
 marker.e &n
endm
; self-rewrite label
srlabel macro n
&n:
endm
        
;end of include z_ram
; defwork,endwork moved to z_ram
beginram
defhook macro h.name
 &h.name:
 jp h.nop
endm
defhook h.unreach
endram
beginram
peekw macro regv,regm
#local
  ld (w-2),&regm
  ld &regv,(0)
srlabel w
#endlocal
endm

pokew macro regm,regv
#local
  ld (w-2),&regm
  ld (0),&regv
srlabel w
#endlocal
endm
endram
defwork xrnd.seed
 dw 1234
 dw 5678
endwork xrnd.seed
defwork hexval
 dw 0
endwork hexval
defwork trad
 db 65
endwork trace.s
defwork cursor
 dw 1800h
endwork cursor
beginram
mem2sp macro &n
 ;(hl)->sp
#local
 ld (rs-2),hl
 ld sp,(0)
srlabel rs
#endlocal
endm

sp2mem macro &n
 ;sp->(hl)
#local
 ld (spad-2),hl
 ld (0),sp
 spad:
#endlocal
endm
endram
defwork sptmp
 dw 0
endwork sptmp
defwork sva
 db 0
endwork sva
beginram
 defhook h.tntimi
endram
defwork timecnt
 db 0
endwork timecnt
defwork th.new.start
 db th.start.h
th.new.end:
 db th.end.h
endwork th.new.start
defwork th.kill.reserve
db 0
endwork th.kill.reserve
beginram
defsub th.yield
 ld hl,(th.cur)
 ld l,th.ofs.sp
 sp2mem 0
srlabel adrssp
 ld sp,0
 jp doint
endsub th.yield
endram
beginram
defsub th.with.s
; with(this) { init routine } 
;  init routine should end with th.with.ret
; h -> this
; th.with.setdst <addr> -> init routine address
 ld (th.wrssp-2),sp
 ld l, th.ofs.sp
 ld (th.updsp-2),hl
 mem2sp 0
 jp 0
srlabel th.jpdest
th.ewith:
 ld (0),sp
srlabel th.updsp
 ld sp,0
srlabel th.wrssp
endsub th.with.s





defsub th.push
 ;push bc to thread hl
 th.with tpsbc
 ret
 tpsbc:
  push bc
  th.with.ret 0
endsub th.push



defsub th.loop
 ; hook before stepall
 call h.thent
 ; save prev timecnt
 ld a,(timecnt)
 push af
  call th.stepall
  ; hook after stepall
  call h.thlop
 ; wait until timecnt changes
 pop af
 bwat:
  ld hl,timecnt
  cp (hl)
  jr nz,bbwat
  push af
  call doint
  pop af
  jr bwat
 bbwat:
 ; repeat
 jr th.loop
endsub th.loop

defhook h.thent
defhook h.thlop

endram
defwork th.cur
 dw 0
endwork th.cur
beginram
; pushthis before arg push
; hl=target
invoketg.a macro fld
 ld a,h
 ld (this),a
 getfld &fld
 callhl
endm
; popthis after

invoke macro fld
 getfld &fld
 callhl 
 getthis 0
endm

getfld.hl macro n
 getfld &n
endm
getfld.de macro n
 ld ($+6),a
 ld de,(&n)
endm

getfld macro n
 ld ($+5),a
 ld hl,(&n)
endm

setfld macro n
 ld ($+5),a
 ld (&n),hl
endm

getfldtg macro n
;hl=tg
 ld l,&n
 peekw hl,hl
endm

setfldtg macro n
; stk=val hl=tg
 ld l,&n
 pop de
 pokew hl,de
endm

tgconst macro n
 ld (&n-1),a
endm
tgconst.g macro r16,n,fld
 ld &r16,(&fld)
srlabel &n
endm
tgconst.s macro n,fld,r16
 ld (&fld),&r16
srlabel &n
endm

defsub h.getfld.hl.rom
 db 2ah; ld hl,(nn)
srlabel getfld.hl.rom.fld
 db 0
srlabel getfld.hl.rom.tg
 db 0
 ret
endsub h.getfld.hl.rom

defsub h.getfld.de.rom
;01RP1011
;01011011
 db 0edh, 5bh; ld de,(nn)
srlabel getfld.de.rom.fld
 db 0
srlabel getfld.de.rom.tg
 db 0
 ret
endsub h.getfld.de.rom

defsub h.setfld.rom
 db 22h ; ld (nn),hl
srlabel setfld.rom.fld
 db 0
srlabel setfld.rom.tg
 db 0
 ret
endsub h.setfld.rom

endram
beginram
defsub o.new
#local
 ; {val .f} n &initbl retad
 pop hl;retad
 ld (retad-2),hl
 ; set initbl for th.with
 pop hl;&initbl
 th.with.setdst hl
 ; save this
 ld (svthis-1),a
 ; allocate thread
 call th.new
 jr nz, allocfail
 push hl; thread address
 call th.with.s; call &initbl
 pop hl; thread address
 ld a,h; set this as thread
 ; init fields
 pop bc; n of {val .f}
 inc c
 loop:
  dec c
  jr z,lpend
  pop hl; .f
  ld h,a
  ld (w-2),hl
  pop hl; val
  ld (w),hl
  srlabel w
 jr loop
 lpend:
 ; return h as this
 ld h,a
 finally:
  ;restore a before call o.new
  ld a,0
  srlabel svthis
  ;ret 
  jp 0
  srlabel retad
 allocfail:
  ; drop {val .f}
  pop bc; n of {val .f}
  ld b,c
  inc c
  lp2:
   dec c
   jr z, lp2end
   pop hl
   pop hl
  jr lp2
  lp2end:
  ld hl,null;  todo null
  jr finally
#endlocal
endsub o.new
endram
defwork this
 db 0
endwork this
beginram
defsub o.boot
 curth2this
 invoke .main
endsub o.boot
endram
beginram
defsub instanceof
 ; a=this de=Class
 ; z: true
 getfld .class
 jp is.subclass.a
endsub instanceof

defsub get.superclass
 ; hl=Class
 dec hl
 dec hl
 peekw hl,hl
endsub get.superclass
endram
beginram


defsub spr.puts
#local
 call spr.putslp
 ld a,(spr.puts.s)
 cp 1
 jr nz,rdir
  ld a, th.count-1
  ld (spr.puts.s),a
  ret
 rdir:
 ld a,1
 ld (spr.puts.s),a

 ld a,(spr.puts.b)
 inc a
 cp th.end.h
 jr c,rsta
  sub th.count
 rsta:
 ld (spr.puts.b),a
#endlocal
endsub spr.puts_new
endram
defwork spr.puts.b
 db th.start.h
endwork spr.puts.b
defwork spr.puts.s
 db 1
endwork spr.puts.s
defwork gbl_offsetY
dw 0
endwork gbl_offsetY
defwork gbl_offsetX
dw 0
endwork gbl_offsetX
beginram
;defsub spr.put1_new
defsub spr.put1
; a: th.id
; uses c
#local

  tgconst t1
  tgconst t2
  tgconst t3
  tgconst t4
  ld h,a
  call th.isblank.a
  ret z
  getthis 0
  
  tgconst.g hl,t1,.y
  ld de,(gbl_offsetY)
  subhl de
  bit 7,h
  jp nz, setsco
  ld de,spr.ymax
  cpde.a 0
  jp nc, setsco
  spr.unscale 0
  sub 8
  ld c,a

  tgconst.g hl,t2,.x
  ld de,(gbl_offsetX)
  subhl de
  bit 7,h
  jp nz, setsco
  ld de,spr.xmax
  cpde.a 0
  jp nc,setsco
  ld a,c
  outwrt 0
  subhl de
  spr.unscale 0
  sub 8
  outwrt 0

  tgconst.g a,t3,.p
  sla a
  sla a
  outwrt 0

  tgconst.g a,t4,.c
  outwrt 0
  getthis 0
  ld h,a
  ld l,.statusReg+1
  res 7,(hl)
  ret 
  
  setsco:
  getthis 0
  ld h,a
  ld l,.statusReg+1
  set 7,(hl)
#endlocal
endsub spr.put1

defsub spr.put1_old
;defsub spr.put1
#local
; a: th.id
  push af

  tgconst t1
  tgconst t2
  tgconst t3
  tgconst t4
  ld h,a
  call th.isblank.a
  jr nz,has
  ; ld a,x is needed
  ; since OpenMSX may reproduce the "nightmare" 
  ; described in https://www.msx.org/wiki/Compatibility_testing
  ; > The real machine is very picky with VDP timing. If you write too fast to the VDP, it might corrupt data.
   ld a,240
   outwrt 0
   ld a,0
   outwrt 0
   ld a,0
   outwrt 0
   ld a,0
   outwrt 0
   pop af
   ret
  has:

  tgconst.g hl,t1,.y 
  spr.unscale 0
  sub 8
  outwrt 0
  
  tgconst.g hl,t2,.x 
  spr.unscale 0
  sub 8
  outwrt 0
  
  tgconst.g a,t3,.p 
  sla a
  sla a
  outwrt 0
  
  tgconst.g a,t4,.c 
  outwrt 0
  pop af
#endlocal
endsub spr.put1_old

endram
defwork keyuse
db 1
endwork keyuse
defwork keymat1
ds 13, 0
endwork keymat1
defwork keymat2
ds 13, 0
endwork keymat2
defwork trig2mat
db 0

;  rdul
db 00100000b
db 10100000b
db 10000000b
db 11000000b
db 01000000b
db 01010000b
db 00010000b
db 00110000b

endwork trig2mat
beginram
MZO.init.ch:
dw 0
MZO.init.mp:
dw 0
endram
beginram
MZO.stop.chn:
dw 0
MZO.stop.ch:
dw 0
endram
beginram
MZO.step.chn:
dw 0
MZO.step.mpoint:
dw 0
MZO.step.mcount:
dw 0
MZO.step.tempoc:
dw 0
MZO.step.len:
dw 0
MZO.step.v:
dw 0
MZO.step.cmd:
dw 0
MZO.step.ofs:
dw 0
MZO.step.ch:
dw 0
endram
beginram
MZO.initBGM.size:
dw 0
MZO.initBGM.mpoint:
dw 0
endram
beginram
MZO.initSE.size:
dw 0
MZO.initSE.mpoint:
dw 0
endram
beginram
MZO.stopBGM.i:
dw 0
endram
beginram
MZO.stepAll.i:
dw 0
endram
beginram
MZOgbl_mpoint:
dw 0, 0, 0, 0
MZOgbl_mcount:
dw 0, 0, 0, 0
MZOgbl_v:
dw 0, 0, 0, 0
MZOgbl_len:
dw 0, 0, 0, 0
MZOgbl_tempoc:
dw 0, 0, 0, 0
MZOgbl_mute:
dw 0, 0, 0, 0;
MZOgbl_psgs:dw 0






endram
beginram

class RActor,Sprite
 RActor.noovr RActor
end.const RActor
RActor.main:
 enter 0
 exit 0
RActor.update:
 invoke .onUpdate
 yield
 ret
RActor.onUpdate:
 ret
RActor.screenOut:
#local
 getfld .x
 ld de,(gbl_offsetX)
 subhl de
 ld de,256 << spr.scale
 subhl de
 jr nc, lb_true
 getfld .y
 ld de,(gbl_offsetY)
 subhl de
 ld de,192 << spr.scale
 subhl de
 jr nc, lb_true
 ld hl,false
 and a
 ret
 lb_true:
 ld hl,true
 scf
 ret
#endlocal
RActor.wait:
#local
 lbl:
 invoke .update
 jr lbl
#endlocal
def RActor.die,0,0
 ld h,a
 push af
 call th.kill.a
 pop af
enddef RActor.die

def RActor.updateEx,1,0
#local
; enter 0
 getarg 1
 n:
  getthis 0
  push hl
  invoke .update
  pop hl
  dec hl
  ld a,h
  or l
 jr nz,n
 getthis 0
#endlocal
enddef RActor.updateEx

crashTo.size equ 8<<spr.scale


defsub crashTo.setXY
 getfld .x
 const cr.gx, hl
 getfld .y
 const cr.gy, hl
endsub crashTo.setXY


def RActor.crashTo,1,0
#local
 call crashTo.setXY
 getarg 1
 ;const cr.class,hl
 call isobj.a
 jr c, cr1
  unreach "c"
  ; "Cannot call target.crashTo(Class) "
 cr1:
  getthis 0
  call crashTo1
  flagtobool c
#endlocal
enddef RActor.crashTo
endram
beginram

defsub crashTo1
 ; call crashTo.setXY before
 ;hl=tg
 ;cy:true
 ;hl is used
 push af
 ld a,h
 tgconst cr.t1
 tgconst cr.t2
 pop af
 tgconst.g hl, cr.t1, .x
 ldconst bc, cr.gx
 subhl bc
 call abs
 ld bc,crashTo.size
 subhl bc
 ret nc

 tgconst.g hl, cr.t2, .y
 ldconst bc, cr.gy
 subhl bc
 call abs
 ld bc,crashTo.size
 subhl bc
endsub crashTo1


endram
defwork gbl_frameCount
 dw 0
endwork gbl_frameCount
beginram
defsub all.die
#local
 pushthis 0
 th.for.a nxt, 0, th.count
  call th.isblank.a
  continue.if z
  invoketg.a .die
  continue
 nxt:
 popthis 0
#endlocal
endsub all.die
endram
defwork strbuf.i
ds 16, 0
endwork strbuf.i
beginram
; class_def_start Laser
;range 0-32
class Laser,RActor
 meth Laser,main
 unuse 0
 fld .y,0
 unuse 0
 unuse 0
 meth RActor,onUpdate
 meth RActor,update
 meth RActor,screenOut
 meth RActor,die
 meth RActor,updateEx
 meth RActor,crashTo
 fld .wat,0
 fld .i,0
 fld .imod,0
 fld .mesg,0
 fld .lim,0
 end.const Laser
def Laser.main,0,0
 #local
 ;wat=5-($scb>>4);
 ld hl,4; const val
 push hl
 ld hl,(gbl_scb)
 pop bc
 ld b,c
 #local
  loop:
  srahl 0
  djnz loop
 #endlocal
 push hl
 ld hl,5; const val
 pop de
 subhl de
 setfld .wat
 ;if wat>0
 ld hl,0; const val
 push hl
 getfld.hl .wat
 pop de
 call hlgtde
 jpf lb1;then
 ;for i=0;
 ;i=0;
 ld hl,0; const val
 setfld .i
 lb2:
 ;while i<12
 ld hl,12; const val
 push hl
 getfld.hl .i
 pop de
 call hlltde
 jpf lb3
 ;imod=i%3;
 getfld.hl .i
 ld de,3; const val
 call div.a
 ex de,hl
 getthis 0
 setfld .imod
 ;sound_playSE(imod==0?$se_a:imod==1?$se_g:$se_f);
 ld hl,0; const val
 push hl
 getfld.hl .imod
 pop de
 call hleqde
 jpf lb5
 ld hl,gbl_se_a; const val
 jp lb6
 lb5:
 ld hl,1; const val
 push hl
 getfld.hl .imod
 pop de
 call hleqde
 jpf lb7
 ld hl,gbl_se_g; const val
 jp lb8
 lb7:
 ld hl,gbl_se_f; const val
 lb8:
 lb6:
 push hl
 call MZO.initSE
 getthis 0
 ;map_set(31,y,peek($lasers[mesg]+imod));
 getfld.hl .y
 push hl
 ld hl,31; const val
 push hl
 getfld.hl .mesg
 add hl,hl
 ld de,gbl_lasers; const val
 add hl,de
 peekw hl,hl
 getfld.de .imod
 add hl, de
 peekw hl,hl
 ld h,0
 ld a,l
 pop hl
 pop de
 call map.set.a
 getthis 0
 ;updateEx(wat);
 getfld.hl .wat
 push hl
 call RActor.updateEx
 getthis 0
 lb4:
 ;next i++
 getfld.hl .i
 inc hl
 setfld .i
 jp lb2
 lb3:
 lb1:
 ;map_set(31,y,$space);
 getfld.hl .y
 push hl
 ld hl,31; const val
 push hl
 ld hl,1; const val
 ld a,l
 pop hl
 pop de
 call map.set.a
 getthis 0
 ;sound_playSE($se_laser);
 ld hl,gbl_se_laser; const val
 push hl
 call MZO.initSE
 getthis 0
 ;for i=31;
 ;i=31;
 ld hl,31; const val
 setfld .i
 lb9:
 ;while i>=0
 ld hl,0; const val
 push hl
 getfld.hl .i
 pop de
 call hlgede
 jpf lb10
 ;if map_get(i,y)==$space
 ld hl,1; const val
 push hl
 getfld.hl .y
 push hl
 getfld.hl .i
 pop de
 call map.get.a
 ld h,0
 ld l,a
 getthis 0
 pop de
 call hleqde
 jpf lb13;then
 ;map_set(i,y,peek($lasers[mesg]+i));
 getfld.hl .y
 push hl
 getfld.hl .i
 push hl
 getfld.hl .mesg
 add hl,hl
 ld de,gbl_lasers; const val
 add hl,de
 peekw hl,hl
 getfld.de .i
 add hl, de
 peekw hl,hl
 ld h,0
 ld a,l
 pop hl
 pop de
 call map.set.a
 getthis 0
 jp lb12
 lb13:;else
 jp lb10
 lb12:
 ;update();
 call RActor.update
 getthis 0
 lb11:
 ;next i--
 getfld.hl .i
 dec hl
 setfld .i
 jp lb9
 lb10:
 ;lim=i;
 getfld.hl .i
 setfld .lim
 ;if lim<0
 ld hl,0; const val
 push hl
 getfld.hl .lim
 pop de
 call hlltde
 jpf lb14;then
 ;lim=0;
 ld hl,0; const val
 setfld .lim
 ;$scm+=2;
 ld hl,(gbl_scm)
 ld de,2; const val
 add hl, de
 ld (gbl_scm),hl
 ;if $scm<96
 ld hl,96; const val
 push hl
 ld hl,(gbl_scm)
 pop de
 call hlltde
 jpf lb16;then
 ;map_set(23-($scm>>2),1,4);
 ld hl,1; const val
 push hl
 ld hl,2; const val
 push hl
 ld hl,(gbl_scm)
 pop bc
 ld b,c
 #local
  loop:
  srahl 0
  djnz loop
 #endlocal
 push hl
 ld hl,23; const val
 pop de
 subhl de
 push hl
 ld hl,4; const val
 ld a,l
 pop hl
 pop de
 call map.set.a
 getthis 0
 jp lb15
 lb16:;else
 ;locate(0,1);
 ld hl,1; const val
 push hl
 ld hl,0; const val
 pop de
 call locate
 ;print($scm);
 ld hl,(gbl_scm)
 call printi.a
 getthis 0
 lb15:
 ;updateEx(30);
 ld hl,30; const val
 push hl
 call RActor.updateEx
 getthis 0
 lb14:
 ;for i=31;
 ;i=31;
 ld hl,31; const val
 setfld .i
 lb17:
 ;while i>=lim
 getfld.hl .lim
 push hl
 getfld.hl .i
 pop de
 call hlgede
 jpf lb18
 ;if map_get(i,y)>=128
 ld hl,128; const val
 push hl
 getfld.hl .y
 push hl
 getfld.hl .i
 pop de
 call map.get.a
 ld h,0
 ld l,a
 getthis 0
 pop de
 call hlgede
 jpf lb20;then
 ;map_set(i,y,$space);
 getfld.hl .y
 push hl
 getfld.hl .i
 push hl
 ld hl,1; const val
 ld a,l
 pop hl
 pop de
 call map.set.a
 getthis 0
 lb20:
 ;update();
 call RActor.update
 getthis 0
 lb19:
 ;next i--
 getfld.hl .i
 dec hl
 setfld .i
 jp lb17
 lb18:
 ;die();
 call RActor.die
 getthis 0
 lbexit:
 #endlocal
 enddef Laser.main
; class_def_end Laser
endram
beginram
; class_def_start LaserR
;range 0-32
class LaserR,RActor
 meth LaserR,main
 unuse 0
 fld .y,0
 unuse 0
 unuse 0
 meth RActor,onUpdate
 meth RActor,update
 meth RActor,screenOut
 meth RActor,die
 meth RActor,updateEx
 meth RActor,crashTo
 fld .wat,0
 fld .i,0
 fld .imod,0
 fld .mesg,0
 fld .lim,0
 end.const LaserR
def LaserR.main,0,0
 #local
 ;wat=5-($scb>>4);
 ld hl,4; const val
 push hl
 ld hl,(gbl_scb)
 pop bc
 ld b,c
 #local
  loop:
  srahl 0
  djnz loop
 #endlocal
 push hl
 ld hl,5; const val
 pop de
 subhl de
 setfld .wat
 ;if wat>0
 ld hl,0; const val
 push hl
 getfld.hl .wat
 pop de
 call hlgtde
 jpf lb21;then
 ;for i=0;
 ;i=0;
 ld hl,0; const val
 setfld .i
 lb22:
 ;while i<12
 ld hl,12; const val
 push hl
 getfld.hl .i
 pop de
 call hlltde
 jpf lb23
 ;imod=i&3;
 getfld.hl .i
 ex de,hl
 ld hl,3; const val
 bitand.a 0
 getthis 0
 setfld .imod
 ;sound_playSE(        imod==0?$se_f:imod==1?$se_g:imod==2?$se_a:$se_b);
 ld hl,0; const val
 push hl
 getfld.hl .imod
 pop de
 call hleqde
 jpf lb25
 ld hl,gbl_se_f; const val
 jp lb26
 lb25:
 ld hl,1; const val
 push hl
 getfld.hl .imod
 pop de
 call hleqde
 jpf lb27
 ld hl,gbl_se_g; const val
 jp lb28
 lb27:
 ld hl,2; const val
 push hl
 getfld.hl .imod
 pop de
 call hleqde
 jpf lb29
 ld hl,gbl_se_a; const val
 jp lb30
 lb29:
 ld hl,gbl_se_b; const val
 lb30:
 lb28:
 lb26:
 push hl
 call MZO.initSE
 getthis 0
 ;map_set(0,y,peek($lasers[mesg]+(i%4)));
 getfld.hl .y
 push hl
 ld hl,0; const val
 push hl
 getfld.hl .i
 ld de,4; const val
 call div.a
 ex de,hl
 getthis 0
 push hl
 getfld.hl .mesg
 add hl,hl
 ld de,gbl_lasers; const val
 add hl,de
 peekw hl,hl
 pop de
 add hl, de
 peekw hl,hl
 ld h,0
 ld a,l
 pop hl
 pop de
 call map.set.a
 getthis 0
 ;updateEx(wat);
 getfld.hl .wat
 push hl
 call RActor.updateEx
 getthis 0
 lb24:
 ;next i++
 getfld.hl .i
 inc hl
 setfld .i
 jp lb22
 lb23:
 lb21:
 ;map_set(0,y,$space);
 getfld.hl .y
 push hl
 ld hl,0; const val
 push hl
 ld hl,1; const val
 ld a,l
 pop hl
 pop de
 call map.set.a
 getthis 0
 ;sound_playSE($se_laser);
 ld hl,gbl_se_laser; const val
 push hl
 call MZO.initSE
 getthis 0
 ;for i=0;
 ;i=0;
 ld hl,0; const val
 setfld .i
 lb31:
 ;while i<32
 ld hl,32; const val
 push hl
 getfld.hl .i
 pop de
 call hlltde
 jpf lb32
 ;if map_get(i,y)==$space
 ld hl,1; const val
 push hl
 getfld.hl .y
 push hl
 getfld.hl .i
 pop de
 call map.get.a
 ld h,0
 ld l,a
 getthis 0
 pop de
 call hleqde
 jpf lb35;then
 ;map_set(i,y,peek($lasers[mesg]+i));
 getfld.hl .y
 push hl
 getfld.hl .i
 push hl
 getfld.hl .mesg
 add hl,hl
 ld de,gbl_lasers; const val
 add hl,de
 peekw hl,hl
 getfld.de .i
 add hl, de
 peekw hl,hl
 ld h,0
 ld a,l
 pop hl
 pop de
 call map.set.a
 getthis 0
 jp lb34
 lb35:;else
 jp lb32
 lb34:
 ;update();
 call RActor.update
 getthis 0
 lb33:
 ;next i++
 getfld.hl .i
 inc hl
 setfld .i
 jp lb31
 lb32:
 ;if i>=32
 ld hl,32; const val
 push hl
 getfld.hl .i
 pop de
 call hlgede
 jpf lb37;then
 ;lim=31;
 ld hl,31; const val
 setfld .lim
 ;$scb+=2;
 ld hl,(gbl_scb)
 ld de,2; const val
 add hl, de
 ld (gbl_scb),hl
 ;if $scb<96
 ld hl,96; const val
 push hl
 ld hl,(gbl_scb)
 pop de
 call hlltde
 jpf lb39;then
 ;map_set(8+($scb>>2),0,5);
 ld hl,0; const val
 push hl
 ld hl,2; const val
 push hl
 ld hl,(gbl_scb)
 pop bc
 ld b,c
 #local
  loop:
  srahl 0
  djnz loop
 #endlocal
 push hl
 ld hl,8; const val
 pop de
 add hl, de
 push hl
 ld hl,5; const val
 ld a,l
 pop hl
 pop de
 call map.set.a
 getthis 0
 jp lb38
 lb39:;else
 ;locate(28,0);
 ld hl,0; const val
 push hl
 ld hl,28; const val
 pop de
 call locate
 ;print($scb);
 ld hl,(gbl_scb)
 call printi.a
 getthis 0
 lb38:
 ;updateEx(30);
 ld hl,30; const val
 push hl
 call RActor.updateEx
 getthis 0
 jp lb36
 lb37:;else
 ;lim=i;
 getfld.hl .i
 setfld .lim
 lb36:
 ;for i=0;
 ;i=0;
 ld hl,0; const val
 setfld .i
 lb40:
 ;while i<=lim
 getfld.hl .lim
 push hl
 getfld.hl .i
 pop de
 call hllede
 jpf lb41
 ;if map_get(i,y)>=128
 ld hl,128; const val
 push hl
 getfld.hl .y
 push hl
 getfld.hl .i
 pop de
 call map.get.a
 ld h,0
 ld l,a
 getthis 0
 pop de
 call hlgede
 jpf lb43;then
 ;map_set(i,y,$space);
 getfld.hl .y
 push hl
 getfld.hl .i
 push hl
 ld hl,1; const val
 ld a,l
 pop hl
 pop de
 call map.set.a
 getthis 0
 lb43:
 ;update();
 call RActor.update
 getthis 0
 lb42:
 ;next i++
 getfld.hl .i
 inc hl
 setfld .i
 jp lb40
 lb41:
 ;die();
 call RActor.die
 getthis 0
 lbexit:
 #endlocal
 enddef LaserR.main
; class_def_end LaserR
endram
beginram
_main:
tnu.run Main,0,32
; class_def_start Main
;range 0-32
class Main,RActor
 meth Main,main
 unuse 0
 fld .y,0
 unuse 0
 unuse 0
 meth RActor,onUpdate
 meth RActor,update
 meth RActor,screenOut
 meth RActor,die
 meth RActor,updateEx
 meth RActor,crashTo
 fld .j,0
 fld .i,0
 meth Main,cls
 fld .mesg,0
 meth Main,demo
 end.const Main
def Main.main,0,0
 #local
 ;locate(0,0);
 ld hl,0; const val
 push hl
 ld hl,0; const val
 pop de
 call locate
 ;cls();
 call Main.cls
 getthis 0
 ;for i=0;
 ;i=0;
 ld hl,0; const val
 setfld .i
 lb44:
 ;while i<8
 ld hl,8; const val
 push hl
 getfld.hl .i
 pop de
 call hlltde
 jpf lb45
 ;new Star{        y=i*45+32,c=15,p=7,vx=(i%3)+2    };
 getfld.hl .i
 ld de,45; const val
 call mul.a
 getthis 0
 ld de,32; const val
 add hl, de
 new.arg .y
 ld hl,15; const val
 new.arg .c
 ld hl,7; const val
 new.arg .p
 getfld.hl .i
 ld de,3; const val
 call div.a
 ex de,hl
 getthis 0
 ld de,2; const val
 add hl, de
 new.arg .vx
 new Star,4,0,32
 lb46:
 ;next i++
 getfld.hl .i
 inc hl
 setfld .i
 jp lb44
 lb45:
 ;if $demo!=12345
 ld hl,12345; const val
 push hl
 ld hl,(gbl_demo)
 pop de
 call hlnede
 jpf lb47;then
 ;demo();
 call Main.demo
 getthis 0
 lb47:
 ;$gov=0;
 ld hl,0; const val
 ld (gbl_gov),hl
 ;sound_playBGM($se_bgms);
 ld hl,gbl_se_bgms; const val
 push hl
 call MZO.initBGM
 getthis 0
 ;new Player;
 new Player,0,0,32
 ;$s=0;
 ld hl,0; const val
 ld (gbl_s),hl
 ;$score=0;
 ld hl,0; const val
 ld (gbl_score),hl
 ;$scb=$scm=0;
 ld hl,0; const val
 ld (gbl_scm),hl
 ld (gbl_scb),hl
 ;locate(1,0);
 ld hl,0; const val
 push hl
 ld hl,1; const val
 pop de
 call locate
 ;print("Bamboo");
 #local
 ld hl,st
 jp nx
 st: db 66, 97, 109, 98, 111, 111, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 ;locate(24,1);
 ld hl,1; const val
 push hl
 ld hl,24; const val
 pop de
 call locate
 ;print("Mushroom");
 #local
 ld hl,st
 jp nx
 st: db 77, 117, 115, 104, 114, 111, 111, 109, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 ;$l=0;
 ld hl,0; const val
 ld (gbl_l),hl
 ;$mb=0;
 ld hl,0; const val
 ld (gbl_mb),hl
 ;$smb=0;
 ld hl,0; const val
 ld (gbl_smb),hl
 lb48:
 ;while !$gov
 ld hl,(gbl_gov)
 call nothl
 jpf lb49
 ;$s--;
 ld hl,(gbl_s)
 dec hl
 ld (gbl_s),hl
 ;if $s<0
 ld hl,0; const val
 push hl
 ld hl,(gbl_s)
 pop de
 call hlltde
 jpf lb50;then
 ;new Stone{dir:$smb==0? -1 :1};
 ld hl,0; const val
 push hl
 ld hl,(gbl_smb)
 pop de
 call hleqde
 jpf lb51
 ld hl,1; const val
 call neghl
 jp lb52
 lb51:
 ld hl,1; const val
 lb52:
 new.arg .dir
 new Stone,1,0,32
 ;$smb=1-$smb;
 ld hl,1; const val
 ld de,(gbl_smb)
 subhl de
 ld (gbl_smb),hl
 ;$s=200+rnd(100);
 ld hl,100; const val
 call rnd
 push hl
 ld hl,200; const val
 pop de
 add hl, de
 ld (gbl_s),hl
 ;locate(1,0);
 ld hl,0; const val
 push hl
 ld hl,1; const val
 pop de
 call locate
 ;print("Bamboo");
 #local
 ld hl,st
 jp nx
 st: db 66, 97, 109, 98, 111, 111, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 lb50:
 ;$l--;
 ld hl,(gbl_l)
 dec hl
 ld (gbl_l),hl
 ;if $l<0
 ld hl,0; const val
 push hl
 ld hl,(gbl_l)
 pop de
 call hlltde
 jpf lb53;then
 ;y=rnd(18)+2;
 ld hl,18; const val
 call rnd
 ld de,2; const val
 add hl, de
 setfld .y
 ;mesg=rnd(2);
 ld hl,2; const val
 call rnd
 setfld .mesg
 ;$score++;
 ld hl,(gbl_score)
 inc hl
 ld (gbl_score),hl
 ;if $mb==0
 ld hl,0; const val
 push hl
 ld hl,(gbl_mb)
 pop de
 call hleqde
 jpf lb55;then
 ;mesg=rnd(10);
 ld hl,10; const val
 call rnd
 setfld .mesg
 ;new Laser{y,mesg};
 getfld.hl .y
 new.arg .y
 getfld.hl .mesg
 new.arg .mesg
 new Laser,2,0,32
 ;new Laser{y:y+1,mesg};
 getfld.hl .y
 ld de,1; const val
 add hl, de
 new.arg .y
 getfld.hl .mesg
 new.arg .mesg
 new Laser,2,0,32
 ;new Laser{y:y+2,mesg};
 getfld.hl .y
 ld de,2; const val
 add hl, de
 new.arg .y
 getfld.hl .mesg
 new.arg .mesg
 new Laser,2,0,32
 ;new Laser{y:y+3,mesg};
 getfld.hl .y
 ld de,3; const val
 add hl, de
 new.arg .y
 getfld.hl .mesg
 new.arg .mesg
 new Laser,2,0,32
 jp lb54
 lb55:;else
 ;mesg=10+rnd(10);
 ld hl,10; const val
 call rnd
 push hl
 ld hl,10; const val
 pop de
 add hl, de
 setfld .mesg
 ;new LaserR{y,mesg};
 getfld.hl .y
 new.arg .y
 getfld.hl .mesg
 new.arg .mesg
 new LaserR,2,0,32
 ;new LaserR{y:y+1,mesg};
 getfld.hl .y
 ld de,1; const val
 add hl, de
 new.arg .y
 getfld.hl .mesg
 new.arg .mesg
 new LaserR,2,0,32
 ;new LaserR{y:y+2,mesg};
 getfld.hl .y
 ld de,2; const val
 add hl, de
 new.arg .y
 getfld.hl .mesg
 new.arg .mesg
 new LaserR,2,0,32
 ;new LaserR{y:y+3,mesg};
 getfld.hl .y
 ld de,3; const val
 add hl, de
 new.arg .y
 getfld.hl .mesg
 new.arg .mesg
 new LaserR,2,0,32
 lb54:
 ;$mb=1-$mb;
 ld hl,1; const val
 ld de,(gbl_mb)
 subhl de
 ld (gbl_mb),hl
 ;$l=100+rnd(30)-(($scm+$scb)>>1);
 ld hl,1; const val
 push hl
 ld hl,(gbl_scm)
 ld de,(gbl_scb)
 add hl, de
 pop bc
 ld b,c
 #local
  loop:
  srahl 0
  djnz loop
 #endlocal
 push hl
 ld hl,30; const val
 call rnd
 push hl
 ld hl,100; const val
 pop de
 add hl, de
 pop de
 subhl de
 ld (gbl_l),hl
 lb53:
 ;update();
 call RActor.update
 getthis 0
 jp lb48
 lb49:
 ;updateEx(60);
 ld hl,60; const val
 push hl
 call RActor.updateEx
 getthis 0
 lb56:
 ;while true
 ld hl,true
 jpf lb57
 ;locate(12,7);
 ld hl,7; const val
 push hl
 ld hl,12; const val
 pop de
 call locate
 ;print("Game\u0001over");
 #local
 ld hl,st
 jp nx
 st: db 71, 97, 109, 101, 1, 111, 118, 101, 114, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 ;locate(10,9);
 ld hl,9; const val
 push hl
 ld hl,10; const val
 pop de
 call locate
 ;print("Continue\u0001Space");
 #local
 ld hl,st
 jp nx
 st: db 67, 111, 110, 116, 105, 110, 117, 101, 1, 83, 112, 97, 99, 101, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 ;if getkey("space")==1
 ld hl,1; const val
 push hl
 ld hl, 264
 call getkey
 pop de
 call hleqde
 jpf lb58;then
 ;loadPage(Main);
 loadPage Main,0,0,32
 lb58:
 ;update();
 call RActor.update
 getthis 0
 jp lb56
 lb57:
 lbexit:
 #endlocal
 enddef Main.main
def Main.cls,0,0
 #local
 ;for i=0;
 ;i=0;
 ld hl,0; const val
 setfld .i
 lb59:
 ;while i<24
 ld hl,24; const val
 push hl
 getfld.hl .i
 pop de
 call hlltde
 jpf lb60
 ;for j=0;
 ;j=0;
 ld hl,0; const val
 setfld .j
 lb62:
 ;while j<32
 ld hl,32; const val
 push hl
 getfld.hl .j
 pop de
 call hlltde
 jpf lb63
 ;map_set(j,i,$space);
 getfld.hl .i
 push hl
 getfld.hl .j
 push hl
 ld hl,1; const val
 ld a,l
 pop hl
 pop de
 call map.set.a
 getthis 0
 lb64:
 ;next j++
 getfld.hl .j
 inc hl
 setfld .j
 jp lb62
 lb63:
 lb61:
 ;next i++
 getfld.hl .i
 inc hl
 setfld .i
 jp lb59
 lb60:
 lbexit:
 #endlocal
 enddef Main.cls
def Main.demo,0,0
 #local
 ;$demo=12345;
 ld hl,12345; const val
 ld (gbl_demo),hl
 ;sound_playBGM($se_iopen);
 ld hl,gbl_se_iopen; const val
 push hl
 call MZO.initBGM
 getthis 0
 ;for i=0;
 ;i=0;
 ld hl,0; const val
 setfld .i
 lb65:
 ;while i<9
 ld hl,9; const val
 push hl
 getfld.hl .i
 pop de
 call hlltde
 jpf lb66
 ;locate(5,i*2+3);
 getfld.hl .i
 ld de,2; const val
 call mul.a
 getthis 0
 ld de,3; const val
 add hl, de
 push hl
 ld hl,5; const val
 pop de
 call locate
 ;printf("%s",$demos[i]);
 getfld.hl .i
 add hl,hl
 ld de,gbl_demos; const val
 add hl,de
 peekw hl,hl
 call print.a
 getthis 0
 getthis 0
 ;if i==8
 ld hl,8; const val
 push hl
 getfld.hl .i
 pop de
 call hleqde
 jpf lb68;then
 jp lb66
 lb68:
 ;for j=0;
 ;j=0;
 ld hl,0; const val
 setfld .j
 lb69:
 ;while j<60&&getkey("space")!=1
 ld hl,60; const val
 push hl
 getfld.hl .j
 pop de
 call hlltde
 andand lb72
 ld hl,1; const val
 push hl
 ld hl, 264
 call getkey
 pop de
 call hlnede
 lb72:
 jpf lb70
 ;rnd(2);
 ld hl,2; const val
 call rnd
 ;update();
 call RActor.update
 getthis 0
 lb71:
 ;next j++
 getfld.hl .j
 inc hl
 setfld .j
 jp lb69
 lb70:
 ;update();
 call RActor.update
 getthis 0
 lb67:
 ;next i++
 getfld.hl .i
 inc hl
 setfld .i
 jp lb65
 lb66:
 lb73:
 ;while getkey("space")!=1
 ld hl,1; const val
 push hl
 ld hl, 264
 call getkey
 pop de
 call hlnede
 jpf lb74
 ;rnd(2);
 ld hl,2; const val
 call rnd
 ;update();
 call RActor.update
 getthis 0
 jp lb73
 lb74:
 ;cls();
 call Main.cls
 getthis 0
 lbexit:
 #endlocal
 enddef Main.demo
; class_def_end Main
endram
beginram
; class_def_start Player
;range 0-32
class Player,RActor
 meth Player,main
 fld .x,0
 fld .y,0
 fld .p,0
 fld .c,0
 meth RActor,onUpdate
 meth RActor,update
 meth RActor,screenOut
 meth RActor,die
 meth RActor,updateEx
 meth RActor,crashTo
 fld .rc,0
 fld .i,0
 end.const Player
def Player.main,0,0
 #local
 ;p=$pat_spr+4;
 ld hl,(gbl_pat_spr)
 ld de,4; const val
 add hl, de
 setfld .p
 ;c=15;
 ld hl,15; const val
 setfld .c
 ;x=200;
 ld hl,200; const val
 setfld .x
 ;y=200;
 ld hl,200; const val
 setfld .y
 ;rc=0;
 ld hl,0; const val
 setfld .rc
 lb75:
 ;while true
 ld hl,true
 jpf lb76
 ;if getkey("left")&&x>16
 ld hl, 4104
 call getkey
 andand lb78
 ld hl,16; const val
 push hl
 getfld.hl .x
 pop de
 call hlgtde
 lb78:
 jpf lb77;then
 ;x-=4;
 getfld.hl .x
 ld de,4; const val
 subhl de
 setfld .x
 lb77:
 ;if getkey("right")&&x<500
 ld hl, 32776
 call getkey
 andand lb80
 ld hl,500; const val
 push hl
 getfld.hl .x
 pop de
 call hlltde
 lb80:
 jpf lb79;then
 ;x+=4;
 getfld.hl .x
 ld de,4; const val
 add hl, de
 setfld .x
 lb79:
 ;if getkey("up")&&y>32
 ld hl, 8200
 call getkey
 andand lb82
 ld hl,32; const val
 push hl
 getfld.hl .y
 pop de
 call hlgtde
 lb82:
 jpf lb81;then
 ;y-=4;
 getfld.hl .y
 ld de,4; const val
 subhl de
 setfld .y
 lb81:
 ;if getkey("down")&&y<360
 ld hl, 16392
 call getkey
 andand lb84
 ld hl,360; const val
 push hl
 getfld.hl .y
 pop de
 call hlltde
 lb84:
 jpf lb83;then
 ;y+=4;
 getfld.hl .y
 ld de,4; const val
 add hl, de
 setfld .y
 lb83:
 ;rc+=(x%3+y%3)+1;
 getfld.hl .y
 ld de,3; const val
 call div.a
 ex de,hl
 getthis 0
 push hl
 getfld.hl .x
 ld de,3; const val
 call div.a
 ex de,hl
 getthis 0
 pop de
 add hl, de
 ld de,1; const val
 add hl, de
 push hl
 getfld.hl .rc
 pop de
 add hl, de
 setfld .rc
 ;if rc>200
 ld hl,200; const val
 push hl
 getfld.hl .rc
 pop de
 call hlgtde
 jpf lb85;then
 ;rc-=rnd(200);
 ld hl,200; const val
 call rnd
 push hl
 getfld.hl .rc
 pop de
 subhl de
 setfld .rc
 lb85:
 ;if map_getAt(x,y)!=$space
 ld hl,1; const val
 push hl
 getfld.hl .y
 push hl
 getfld.hl .x
 pop de
 call map.getat.a
 ld h,0
 ld l,a
 getthis 0
 pop de
 call hlnede
 jpf lb86;then
 jp lb76
 lb86:
 ;update();
 call RActor.update
 getthis 0
 jp lb75
 lb76:
 ;sound_playBGM($se_miss);
 ld hl,gbl_se_miss; const val
 push hl
 call MZO.initBGM
 getthis 0
 ;for rc=0;
 ;rc=0;
 ld hl,0; const val
 setfld .rc
 lb87:
 ;while rc<20
 ld hl,20; const val
 push hl
 getfld.hl .rc
 pop de
 call hlltde
 jpf lb88
 ;for i=0;
 ;i=0;
 ld hl,0; const val
 setfld .i
 lb90:
 ;while i<3
 ld hl,3; const val
 push hl
 getfld.hl .i
 pop de
 call hlltde
 jpf lb91
 ;p=$pat_spr+8+i;
 ld hl,(gbl_pat_spr)
 ld de,8; const val
 add hl, de
 getfld.de .i
 add hl, de
 setfld .p
 ;updateEx(3);
 ld hl,3; const val
 push hl
 call RActor.updateEx
 getthis 0
 lb92:
 ;next i++
 getfld.hl .i
 inc hl
 setfld .i
 jp lb90
 lb91:
 lb89:
 ;next rc++
 getfld.hl .rc
 inc hl
 setfld .rc
 jp lb87
 lb88:
 ;$gov=1;
 ld hl,1; const val
 ld (gbl_gov),hl
 lbexit:
 #endlocal
 enddef Player.main
; class_def_end Player
endram
beginram
; class_def_start Star
;range 0-32
class Star,RActor
 meth Star,main
 fld .x,0
 unuse 0
 unuse 0
 unuse 0
 meth RActor,onUpdate
 meth RActor,update
 meth RActor,screenOut
 meth RActor,die
 meth RActor,updateEx
 meth RActor,crashTo
 fld .vx,0
 end.const Star
def Star.main,0,0
 #local
 ;x=rnd(512);
 ld hl,512; const val
 call rnd
 setfld .x
 lb93:
 ;while 1
 ld hl,1; const val
 jpf lb94
 ;x-=vx;
 getfld.hl .x
 getfld.de .vx
 subhl de
 setfld .x
 ;if x<0
 ld hl,0; const val
 push hl
 getfld.hl .x
 pop de
 call hlltde
 jpf lb95;then
 ;x=512 ;
 ld hl,512; const val
 setfld .x
 lb95:
 ;update();
 call RActor.update
 getthis 0
 jp lb93
 lb94:
 lbexit:
 #endlocal
 enddef Star.main
; class_def_end Star
endram
beginram
; class_def_start Stone
;range 0-32
class Stone,RActor
 meth Stone,main
 fld .x,0
 fld .y,0
 unuse 0
 unuse 0
 meth RActor,onUpdate
 meth RActor,update
 meth RActor,screenOut
 meth RActor,die
 meth RActor,updateEx
 meth RActor,crashTo
 fld .vy,0
 fld .i,0
 fld .xc,0
 fld .dir,0
 meth Stone,erase
 meth Stone,put
 end.const Stone
def Stone.main,0,0
 #local
 ;y=rnd(10)+10;
 ld hl,10; const val
 call rnd
 ld de,10; const val
 add hl, de
 setfld .y
 ;vy=rnd(2)?1:-1;
 ld hl,2; const val
 call rnd
 jpf lb96
 ld hl,1; const val
 jp lb97
 lb96:
 ld hl,1; const val
 call neghl
 lb97:
 setfld .vy
 ;xc=0;
 ld hl,0; const val
 setfld .xc
 ;if dir<0
 ld hl,0; const val
 push hl
 getfld.hl .dir
 pop de
 call hlltde
 jpf lb99;then
 ;x=28;
 ld hl,28; const val
 setfld .x
 jp lb98
 lb99:;else
 ;x=0;
 ld hl,0; const val
 setfld .x
 lb98:
 lb100:
 ;while true
 ld hl,true
 jpf lb101
 ;erase();
 call Stone.erase
 getthis 0
 ;if xc
 getfld.hl .xc
 jpf lb102;then
 ;x+=dir;
 getfld.hl .x
 getfld.de .dir
 add hl, de
 setfld .x
 lb102:
 ;xc=1-xc;
 ld hl,1; const val
 getfld.de .xc
 subhl de
 setfld .xc
 ;if x<0||x>28
 ld hl,0; const val
 push hl
 getfld.hl .x
 pop de
 call hlltde
 oror lb104
 ld hl,28; const val
 push hl
 getfld.hl .x
 pop de
 call hlgtde
 lb104:
 jpf lb103;then
 jp lb101
 lb103:
 ;y+=vy;
 getfld.hl .y
 getfld.de .vy
 add hl, de
 setfld .y
 ;put();
 call Stone.put
 getthis 0
 ;if y>17
 ld hl,17; const val
 push hl
 getfld.hl .y
 pop de
 call hlgtde
 jpf lb105;then
 ;vy=-1;
 ld hl,1; const val
 call neghl
 setfld .vy
 lb105:
 ;if y<3
 ld hl,3; const val
 push hl
 getfld.hl .y
 pop de
 call hlltde
 jpf lb106;then
 ;vy=1;
 ld hl,1; const val
 setfld .vy
 lb106:
 ;updateEx(12);
 ld hl,12; const val
 push hl
 call RActor.updateEx
 getthis 0
 jp lb100
 lb101:
 ;die();
 call RActor.die
 getthis 0
 lbexit:
 #endlocal
 enddef Stone.main
def Stone.erase,0,0
 #local
 ;for i=0;
 ;i=0;
 ld hl,0; const val
 setfld .i
 lb107:
 ;while i<5
 ld hl,5; const val
 push hl
 getfld.hl .i
 pop de
 call hlltde
 jpf lb108
 ;locate(x,y+i);
 getfld.hl .y
 getfld.de .i
 add hl, de
 push hl
 getfld.hl .x
 pop de
 call locate
 ;print("\u0001\u0001\u0001\u0001");
 #local
 ld hl,st
 jp nx
 st: db 1, 1, 1, 1, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 lb109:
 ;next i++
 getfld.hl .i
 inc hl
 setfld .i
 jp lb107
 lb108:
 lbexit:
 #endlocal
 enddef Stone.erase
def Stone.put,0,0
 #local
 ;if dir>0
 ld hl,0; const val
 push hl
 getfld.hl .dir
 pop de
 call hlgtde
 jpf lb111;then
 ;locate(x+1,y+0);
 ld hl,0; const val
 push hl
 getfld.hl .y
 pop de
 add hl, de
 push hl
 getfld.hl .x
 ld de,1; const val
 add hl, de
 pop de
 call locate
 ;print(    "\u000d\u000e");
 #local
 ld hl,st
 jp nx
 st: db 13, 14, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 ;locate(x+1,y+1);
 getfld.hl .y
 ld de,1; const val
 add hl, de
 push hl
 getfld.hl .x
 ld de,1; const val
 add hl, de
 pop de
 call locate
 ;print(    "\u0015\u0016");
 #local
 ld hl,st
 jp nx
 st: db 21, 22, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 ;locate(x,y+2);
 getfld.hl .y
 ld de,2; const val
 add hl, de
 push hl
 getfld.hl .x
 pop de
 call locate
 ;print("\u001c\u001d\u001e\u001f");
 #local
 ld hl,st
 jp nx
 st: db 28, 29, 30, 31, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 ;locate(x,y+3);
 getfld.hl .y
 ld de,3; const val
 add hl, de
 push hl
 getfld.hl .x
 pop de
 call locate
 ;print("\u0024\u0025\u0026\u0027");
 #local
 ld hl,st
 jp nx
 st: db 36, 37, 38, 39, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 ;locate(x,y+4);
 getfld.hl .y
 ld de,4; const val
 add hl, de
 push hl
 getfld.hl .x
 pop de
 call locate
 ;print("\u002c\u002d\u002e\u002f");
 #local
 ld hl,st
 jp nx
 st: db 44, 45, 46, 47, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 jp lb110
 lb111:;else
 ;locate(x+1,y+0);
 ld hl,0; const val
 push hl
 getfld.hl .y
 pop de
 add hl, de
 push hl
 getfld.hl .x
 ld de,1; const val
 add hl, de
 pop de
 call locate
 ;print(      "\u0009\u000a");
 #local
 ld hl,st
 jp nx
 st: db 9, 10, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 ;locate(x+0,y+1);
 getfld.hl .y
 ld de,1; const val
 add hl, de
 push hl
 ld hl,0; const val
 push hl
 getfld.hl .x
 pop de
 add hl, de
 pop de
 call locate
 ;print("\u0010\u0011\u0012\u0013");
 #local
 ld hl,st
 jp nx
 st: db 16, 17, 18, 19, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 ;locate(x,y+2);
 getfld.hl .y
 ld de,2; const val
 add hl, de
 push hl
 getfld.hl .x
 pop de
 call locate
 ;print("\u0018\u0019\u001a\u001b");
 #local
 ld hl,st
 jp nx
 st: db 24, 25, 26, 27, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 ;locate(x+1,y+3);
 getfld.hl .y
 ld de,3; const val
 add hl, de
 push hl
 getfld.hl .x
 ld de,1; const val
 add hl, de
 pop de
 call locate
 ;print("\u0021\u0022");
 #local
 ld hl,st
 jp nx
 st: db 33, 34, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 ;locate(x+1,y+4);
 getfld.hl .y
 ld de,4; const val
 add hl, de
 push hl
 getfld.hl .x
 ld de,1; const val
 add hl, de
 pop de
 call locate
 ;print("\u0029\u002a");
 #local
 ld hl,st
 jp nx
 st: db 41, 42, 0
 nx:
 #endlocal
 call print.a
 getthis 0
 lb110:
 lbexit:
 #endlocal
 enddef Stone.put
; class_def_end Stone
endram
beginram
gbl_players:
ds 8,0
gbl_colors:
dw 9, 7, 3, 11
gbl_scb:dw 0
gbl_scm:dw 0
gbl_demo:dw 0
gbl_gov:dw 0
gbl_s:dw 0
gbl_score:dw 0
gbl_l:dw 0
gbl_mb:dw 0
gbl_smb:dw 0
gbl_pat_spr:dw 0
gbl_mesg:
db 72, 101, 108, 108, 111, 44, 32, 119, 111, 114, 108, 100, 0
gbl_mesgs:
dw gbl_mesgs_0
dw gbl_mesgs_1
gbl_mesgs_0:
db 72, 101, 108, 108, 111, 0
gbl_mesgs_1:
db 87, 111, 114, 108, 100, 33, 33, 0
gbl_lasers:
dw gbl_lasers_0
dw gbl_lasers_1
dw gbl_lasers_2
dw gbl_lasers_3
dw gbl_lasers_4
dw gbl_lasers_5
dw gbl_lasers_6
dw gbl_lasers_7
dw gbl_lasers_8
dw gbl_lasers_9
dw gbl_lasers_10
dw gbl_lasers_11
dw gbl_lasers_12
dw gbl_lasers_13
dw gbl_lasers_14
dw gbl_lasers_15
dw gbl_lasers_16
dw gbl_lasers_17
dw gbl_lasers_18
dw gbl_lasers_19
gbl_lasers_0:
db 151, 233, 154, 239, 156, 222, 227, 222, 224, 222, 146, 157, 151, 151, 233, 154
db 239, 156, 222, 227, 222, 149, 156, 227, 222, 157, 151, 233, 154, 228, 227, 243
db 149, 157, 157, 242, 151, 233, 154, 239, 156, 222, 227, 222, 149, 146, 156, 146
db 0
gbl_lasers_1:
db 151, 233, 154, 224, 222, 146, 157, 151, 151, 233, 154, 158, 222, 143, 224, 146
db 149, 156, 227, 222, 157, 151, 233, 154, 158, 222, 143, 224, 146, 155, 146, 154
db 147, 151, 233, 154, 234, 239, 249, 0
gbl_lasers_2:
db 151, 233, 154, 228, 227, 243, 234, 239, 249, 151, 233, 154, 150, 222, 225, 227
db 222, 155, 146, 154, 147, 151, 233, 154, 228, 227, 243, 149, 156, 227, 222, 157
db 151, 233, 154, 158, 222, 143, 224, 146, 234, 239, 249, 0
gbl_lasers_3:
db 151, 233, 154, 239, 156, 222, 227, 222, 234, 239, 249, 151, 233, 154, 228, 227
db 243, 234, 239, 249, 151, 233, 154, 158, 222, 143, 224, 146, 146, 224, 222, 146
db 151, 233, 154, 158, 222, 143, 224, 146, 149, 156, 227, 222, 157, 0
gbl_lasers_4:
db 151, 233, 154, 224, 222, 146, 157, 151, 151, 233, 154, 158, 222, 143, 224, 146
db 155, 146, 154, 147, 151, 233, 154, 228, 227, 243, 234, 239, 249, 151, 233, 154
db 158, 222, 143, 224, 146, 149, 146, 156, 146, 0
gbl_lasers_5:
db 151, 233, 154, 157, 154, 222, 152, 155, 146, 154, 147, 151, 233, 154, 150, 222
db 225, 227, 222, 148, 247, 146, 151, 233, 154, 157, 154, 222, 152, 149, 156, 227
db 222, 157, 151, 233, 154, 150, 222, 225, 227, 222, 234, 239, 249, 0
gbl_lasers_6:
db 151, 233, 154, 242, 143, 225, 140, 149, 146, 156, 146, 151, 233, 154, 228, 227
db 243, 155, 146, 154, 147, 151, 233, 154, 239, 156, 222, 227, 222, 146, 224, 222
db 146, 151, 233, 154, 150, 222, 225, 227, 222, 147, 239, 146, 0
gbl_lasers_7:
db 151, 233, 154, 158, 222, 143, 224, 146, 234, 239, 249, 151, 233, 154, 242, 143
db 225, 140, 234, 239, 249, 151, 233, 154, 242, 143, 225, 140, 234, 239, 249, 151
db 233, 154, 150, 222, 225, 227, 222, 146, 224, 222, 146, 0
gbl_lasers_8:
db 151, 233, 154, 239, 156, 222, 227, 222, 148, 247, 146, 151, 233, 154, 242, 143
db 225, 140, 224, 222, 146, 157, 151, 151, 233, 154, 228, 227, 243, 148, 247, 146
db 151, 233, 154, 239, 156, 222, 227, 222, 149, 156, 227, 222, 157, 0
gbl_lasers_9:
db 151, 233, 154, 147, 239, 146, 151, 233, 154, 242, 143, 225, 140, 155, 146, 154
db 147, 151, 233, 154, 147, 239, 146, 151, 233, 154, 149, 157, 157, 242, 151, 233
db 154, 150, 222, 225, 227, 222, 146, 224, 222, 146, 0
gbl_lasers_10:
db 224, 153, 233, 154, 147, 239, 146, 224, 153, 233, 154, 150, 222, 225, 227, 222
db 234, 239, 249, 224, 153, 233, 154, 150, 222, 225, 227, 222, 149, 146, 156, 146
db 224, 153, 233, 154, 242, 143, 225, 140, 224, 222, 146, 157, 151, 0
gbl_lasers_11:
db 224, 153, 233, 154, 157, 154, 222, 152, 155, 146, 154, 147, 224, 153, 233, 154
db 242, 143, 225, 140, 148, 247, 146, 224, 153, 233, 154, 150, 222, 225, 227, 222
db 234, 239, 249, 0
gbl_lasers_12:
db 224, 153, 233, 154, 242, 143, 225, 140, 224, 222, 146, 157, 151, 224, 153, 233
db 154, 239, 156, 222, 227, 222, 155, 146, 154, 147, 224, 153, 233, 154, 150, 222
db 225, 227, 222, 147, 239, 146, 0
gbl_lasers_13:
db 224, 153, 233, 154, 150, 222, 225, 227, 222, 146, 224, 222, 146, 224, 153, 233
db 154, 242, 143, 225, 140, 155, 146, 154, 147, 224, 153, 233, 154, 239, 156, 222
db 227, 222, 149, 146, 156, 146, 0
gbl_lasers_14:
db 224, 153, 233, 154, 150, 222, 225, 227, 222, 234, 239, 249, 224, 153, 233, 154
db 242, 143, 225, 140, 234, 239, 249, 224, 153, 233, 154, 158, 222, 143, 224, 146
db 224, 222, 146, 157, 151, 0
gbl_lasers_15:
db 224, 153, 233, 154, 228, 227, 243, 149, 156, 227, 222, 157, 224, 153, 233, 154
db 239, 156, 222, 227, 222, 224, 222, 146, 157, 151, 224, 153, 233, 154, 147, 239
db 146, 224, 153, 233, 154, 157, 154, 222, 152, 149, 156, 227, 222, 157, 0
gbl_lasers_16:
db 224, 153, 233, 154, 157, 154, 222, 152, 234, 239, 249, 224, 153, 233, 154, 150
db 222, 225, 227, 222, 149, 146, 156, 146, 224, 153, 233, 154, 150, 222, 225, 227
db 222, 149, 157, 157, 242, 0
gbl_lasers_17:
db 224, 153, 233, 154, 228, 227, 243, 155, 146, 154, 147, 224, 153, 233, 154, 157
db 154, 222, 152, 149, 157, 157, 242, 224, 153, 233, 154, 158, 222, 143, 224, 146
db 149, 157, 157, 242, 0
gbl_lasers_18:
db 224, 153, 233, 154, 158, 222, 143, 224, 146, 148, 247, 146, 224, 153, 233, 154
db 239, 156, 222, 227, 222, 155, 146, 154, 147, 224, 153, 233, 154, 147, 239, 146
db 224, 153, 233, 154, 157, 154, 222, 152, 149, 157, 157, 242, 0
gbl_lasers_19:
db 224, 153, 233, 154, 239, 156, 222, 227, 222, 146, 224, 222, 146, 224, 153, 233
db 154, 234, 239, 249, 224, 153, 233, 154, 228, 227, 243, 146, 224, 222, 146, 224
db 153, 233, 154, 150, 222, 225, 227, 222, 155, 146, 154, 147, 0
gbl_demos:
dw gbl_demos_0
dw gbl_demos_1
dw gbl_demos_2
dw gbl_demos_3
dw gbl_demos_4
dw gbl_demos_5
dw gbl_demos_6
dw gbl_demos_7
dw gbl_demos_8
gbl_demos_0:
db 252, 250, 252, 250, 234, 1, 225, 151, 141, 147, 150, 222, 146, 1, 158, 146
db 242, 146, 224, 146, 228, 233, 0
gbl_demos_1:
db 1, 1, 1, 158, 253, 159, 147, 230, 1, 145, 153, 152, 250, 227, 146, 224
db 0
gbl_demos_2:
db 1, 1, 226, 146, 230, 1, 252, 150, 222, 1, 158, 253, 228, 147, 151, 234
db 0
gbl_demos_3:
db 1, 227, 151, 233, 1, 225, 141, 147, 157, 147, 233, 1, 146, 248, 152, 222
db 225, 230, 0
gbl_demos_4:
db 1, 1, 1, 224, 228, 222, 248, 226, 152, 154, 228, 150, 222, 227, 222, 151
db 224, 0
gbl_demos_5:
db 1, 1, 156, 150, 156, 1, 252, 250, 252, 250, 234, 1, 156, 247, 229, 150
db 143, 224, 0
gbl_demos_6:
db 1, 1, 246, 248, 1, 156, 253, 154, 152, 229, 162, 158, 253, 159, 147, 163
db 230, 0
gbl_demos_7:
db 1, 146, 226, 233, 239, 230, 150, 1, 239, 151, 154, 239, 250, 227, 1, 146
db 224, 154, 228, 134, 0
gbl_demos_8:
db 1, 1, 1, 80, 114, 101, 115, 115, 1, 83, 112, 97, 99, 101, 1, 75
db 101, 121, 0

endram
end _main